images#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : 重定向
# @Time         : 2024/12/3 15:41
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 

from meutils.pipe import *

"gpt-3.5-turbo,gpt-3.5-turbo-0125,gpt-3.5-turbo-0301,gpt-3.5-turbo-0613,gpt-3.5-turbo-1106,gpt-3.5-turbo-16k,gpt-3.5-turbo-16k-0613,gpt-3.5-turbo-instruct"

"""
gpt-4o-2024-08-06,chatgpt-4o-latest,
gpt-4o,gpt-4o-2024-05-13,
gpt-4,gpt-4-0125-preview,gpt-4-0314,gpt-4-0613,gpt-4-1106-preview,gpt-4-32k,gpt-4-32k-0314,gpt-4-32k-0613,gpt-4-turbo,gpt-4-turbo-2024-04-09,gpt-4-turbo-preview
"""

x = """
             
视频生成模型操作指南

- 视频生成能力对prompt提示词撰写的精细度与准确度要求较高，建议大家可以反复细化、调整、优化prompt，实现你的想法；
- AI大模型每次生成的内容具有不确定性，生成结果未达到100%可控生成，比如同时输入多个实体，可能不会都生成出来；颜色和数量可能未必匹配；同样的prompt提示词未必每次生成的回答或图片都一样，建议多次尝试；
- 示例与提示词仅供参考，建议大家撰写稿件、体验时自行探索。
- 此次为小范围体验，（1）超写实画质：模型生成的视频内容具备高清质感、真实感，可用于工业级商业场景例如广告宣传、创意视频生成等商业应用。（2）高语义一致：用户可以进行细致的刻画，例如生成主体的细节，人物概念的组合等。模型可以准确的表达出文本的内容。（3）运动画面流畅：可生成大幅度的合理运动，运动镜头流畅、符合物理规律，不易出现变形。（4）原生镜头转换：模型原生具备自动生成多视角同主体的镜头切换画面，增强画面叙事感。
- 示例与提示词仅供参考，建议大家撰写稿件、体验时自行探索。

注：为了给大家提供更好的服务体验，混元大模型仍在升级和调试中，使用过程中或可能有抖动，欢迎大家反馈问题，一起让它变得更好。
混元视频生成大模型体验入口
正式上线后，用户通过腾讯元宝APP-AI应用-AI视频即可使用该功能（前期需申请）
文生视频提示词撰写注意事项
输入一段文字，腾讯混元视频生成大模型可根据文本生成对应画面， 支持中文、英文输入，生成5秒视频，选择多种画幅。建议对你想要的画面进行尽可能详细的描述，并多次调整你的提示词，以达到最优画面，如果您是专业的影视艺术工作者，可以尝试用专业方式描述画面。如果你为非专业人士，建议可以参考以下格式撰写关键词：
用法1： 提示词 = 主体+场景+运动 
用法2： 提示词=主体（主体描述）+场景（场景描述）+运动（运动描述）+（镜头语言）+（氛围描述）+（风格表达）
用法3： 提示词 = 主体+场景+运动 + (风格表达)  + (氛围描述) + (运镜方式) + (光线) + (景别)

暂时无法在飞书文档外展示此内容

对画面进行更详细，精确的描述，会让生成的视频更加符合你的想法，并且进一步提升画面质感与细节。
提示词ver1：一个人坐在沙发上看书。
一个人坐在沙发上看书。-2.mp4

提示词ver2：一位戴着眼镜的老妇人坐在舒适的沙发上，全神贯注地阅读着一本书。一个冒着热气的茶杯放在咖啡桌上。房间里有书架，书架上放满书籍。
一位戴着眼镜的老妇人坐在舒适的沙发上，全神贯注地阅读着一本书。一个冒着热气的茶杯放在咖啡桌上。房间里有书架，书架上放满书籍。.mp4

提示词ver3：镜头慢慢推近。背景是一个小巧而温馨的客厅。一位年轻女子坐在舒适的沙发上，全神贯注地阅读着一本书。一个冒着热气的茶杯放在咖啡桌上，周围散落着一些靠垫。房间里摆满了植物和书架，给人一种温暖和宁静的感觉。昏暗的中景，舒适的客厅。现实主义风格。
镜头慢慢推近。背景是一个小巧而温馨的客厅。一位年轻女子坐在舒适的沙发上，全神贯注地阅读着一本书。一个冒着热气的茶杯放在咖啡桌上，周围散落着一些靠垫。房间里摆满了.mp4

优势能力&部分案例展示
以下为提示词以及生成的视频
（本文中相关视频均可各位媒体老师作为素材下载，可直接点开相应视频后点击右下角下载）
超写实质感
200mm长焦镜头，对准一只停在栏杆上的海鸥。羽毛的纹理清晰可辨，前景的木栏杆温柔虚化。远处的落日和海平面融化成金红色的光影
200mm长焦镜头，对准一只停在栏杆上的海鸥。羽毛的纹理清晰可辨，前景的木栏杆温柔虚化。远处的落日和海平面融化成金红色的光影.mp4

一个隐藏的瀑布流入清澈的池塘，周围是高大的树木和绿色植物。阳光穿过树叶，在下面的岩石和水面上形成斑点。逼真，茂密森林中的宁静瀑布，宁静、阳光明媚的日子，中景
一个隐藏的瀑布流入清澈的池塘，周围是高大的树木和绿色植物。阳光穿过树叶，在下面的岩石和水面上形成斑点。逼真，茂密森林中的宁静瀑布，宁静、阳光明媚的日子，中景.mp4

超广角仰拍，银河横跨夜空如璀璨星河。远处山脉轮廓若隐若现，篝火在前景点亮帐篷。人类与宇宙的静谧对话。
超广角仰拍.mov

暴风雪中，一列蒸汽火车在崎岖山间穿行，黑烟从车头直冲云霄，车厢在皑皑白雪中留下深邃轨迹，镜头以侧面追踪，捕捉机械巨兽破开风雪的磅礴气势，白雾与蒸汽交织成壮丽画卷，电影氛围，远景
暴风雪中，一列蒸汽火车在崎岖山间穿行，黑烟从车头直冲云霄，车厢在皑皑白雪中留下深邃轨迹，镜头以侧面追踪，捕捉机械巨兽破开风雪的磅礴气势，白雾与蒸汽交织成壮丽画卷，电影氛围，远景.mp4

低角度推进镜头，全景深。清晨，薄雾缭绕山间，红袍僧侣缓步前行，身影与远山轮廓相映。背景古木参天，阳光穿透云层洒落，形成光束。整体色调宁静，突出自然与人文的和谐共生。
低角度推进镜头，全景深.mov

水下广角，阳光从破损甲板射入形成光束阵列。鱼群穿过船舱窗户，铁锈和珊瑚共同装点残骸。历史与海洋生命交织的沉默诗篇。
水下广角，阳光从破损甲板射入形成光束阵列.mov

超长焦横移，工业废弃厂房，主光从碎裂天窗渗入，灰尘颗粒漂浮，人物剪影走过生锈机器，深褐色调
超长焦横移，工业废弃厂房，主光从碎裂天窗渗入.mov

对称构图蜂巢正中特写，单一顶光打出完美几何光影，蜜蜂群以严格对称队形飞出，每只蜂翼的频率形成视觉韵律，空间中的光线被切割成规则图案。
对称构图蜂巢正中特写，单一顶光打出完美几何光影.mov

高语义一致
一位戴着复古飞行护目镜的机械师，半跪在蒸汽朋克风格的工作室里。她棕色卷发挽成发髻，零星的银色发丝闪着金属光泽。深棕色皮质工装背带裤上沾满机油污渍，袖口卷起露出布满齿轮纹身的手臂。特写她正用黄铜工具调试一只机械鸟，齿轮间冒出缕缕蒸汽，工作台上散落着铜管、发条和老式图纸。
一位戴着复古飞行护目镜的机械师，半跪在蒸汽朋克风格的工作室里。她棕色卷发挽成发髻，零星的银色发丝闪着金属光泽。深棕色皮质工装背带裤上沾满机油污渍，袖口卷起露出布满齿轮纹身的手臂。特写她正用黄铜工具调试一只机械鸟，齿轮间冒出缕缕蒸汽，工作台上散落着铜管、发条和老式图纸。.mp4

固定机位的老公寓内景，自然光透过纱帘漫射，青色街灯渗入，茶烟袅袅升起，老式家具静静陈列，定格岁月流逝的时光。
语义一致.mov

45度俯视固定镜头，中景浅景深，晨光穿过梧桐斑驳洒落。画面中央的橘猫围着围裙，稳稳举刀切黄瓜，尾巴随着切菜节奏轻晃。背景灶台炊烟缭绕上升，阳光斜射在木案与老铁锅上泛起温暖光晕。
45度俯视固定镜头，中景浅景深，晨光穿过梧桐斑驳洒落。画面中央的橘猫围着围裙，稳稳举刀切黄瓜，尾巴随着切菜节奏轻晃。背景灶台炊烟缭绕上升，阳光斜射在木案与老铁锅上泛起温暖光晕。.mp4

90度俯瞰镜头，浅景深聚焦。画面中央的芭蕾舞者着白色轻纱，足尖点地旋转。裙摆如同绽放的睡莲，在水晶地面上映出万花筒般的倒影。特写她指尖划过空气，带起细微的波纹，每个动作都在光影间若隐若现。背景的丝绒幕布随气流轻轻波动，勾勒出完美的圆形构图。
芭蕾舞者.mp4

5度斜角固定镜头，浅景深对焦，紫红霓虹灯与青色全息投影交织。画面中央的机械舞者着装前卫，钛合金骨架上覆盖着半透明纳米纤维，每个关节都镶嵌着脉动的LED光环。她缓缓展开双臂，背部的石墨烯光翼像液态金属般流淌展开，翼尖闪烁着等离子般的蓝色电弧。
5度斜角固定镜头，浅景深对焦，紫红霓虹灯与青色全息投影交织。画面中央的机械舞者着装前卫，钛合金骨架上覆盖着半透明纳米纤维，每个关节都镶嵌着脉动的LED光环。她缓缓展开双臂，背部的石墨烯光翼像液态金属般流淌展开，翼尖闪烁着等离子般的蓝色电弧。.mp4

运动画面流畅

超大海浪，冲浪者在浪花上起跳，完成空中转体。摄影机从海浪内部穿越而出，捕捉阳光透过海水的瞬间。水花在空中形成完美弧线，冲浪板划过水面留下轨迹。最后定格在冲浪者穿越水帘的完美瞬间。
冲浪.mp4

穿着白床单的幽灵面对着镜子。镜子中可以看到幽灵的倒影。幽灵位于布满灰尘的阁楼中，阁楼里有老旧的横梁和被布料遮盖的家具。阁楼的场景映照在镜子中。幽灵在镜子前跳舞。电影氛围，电影打光
跳舞的幽灵.mp4

雨滴敲打着窗户。整个视频呈现出悲伤的氛围。
雨滴.mp4

草莓掉进鸡尾酒杯，溅起水花，高速摄影，2K
鸡尾酒.mp4

日落时分的越野赛道，改装过的福特F-150猛禽呼啸而过。加高的悬挂让硕大的防爆轮胎在泥地上肆意翻飞，泥浆飞溅在防滚架上形成斑驳图案。车身贴花在金色阳光下闪闪发亮，机械增压器的呼啸声与排气轰鸣交织。特写车轮陷入松软泥地又瞬间爆发，带起一串泥浆礼花。
日落时分的越野赛道，改装过的福特F-150猛禽呼啸而过。加高的悬挂让硕大的防爆轮胎在泥地上肆意翻飞，泥浆飞溅在防滚架上形成斑驳图案。车身贴花在金色阳光下闪闪发亮，机械增压器的呼啸声与排气轰鸣交织。特写车轮陷入松软泥地又瞬间爆发，带起一串泥浆礼花。.mp4

平移跟摄视角缓缓推进，景深中焦，暖色夕阳光线笼罩画面。画面中的女孩裙摆飘动奔跑，转身跃起，身影与橙红晚霞交织。背景海浪持续轻拍沙滩，洒落金色水花。
平移跟摄视角缓缓推进，景深中焦，暖色夕阳光线笼罩画面。画面中的女孩裙摆飘动奔跑，转身跃起，身影与橙红晚霞交织。背景海浪持续轻拍沙滩，洒落金色水花。.mp4

长焦镜头穿透漫天黄沙，捕捉远处黎明沙丘的剪影。画面中央，一只体态圆润的大熊猫正缓缓展开太极动作，黑白皮毛在旋转的沙尘中若隐若现。摄影机低角度环绕，熊猫毛发随风舞动，每个动作都与席卷的沙浪完美融合。逆光中，它的身影与起伏的沙丘勾勒出东方水墨意境。
长焦镜头穿透漫天黄沙，捕捉远处黎明沙丘的剪影。画面中央，一只体态圆润的大熊猫正缓缓展开太极动作，黑白皮毛在旋转的沙尘中若隐若现。摄影机低角度环绕，熊猫毛发随风舞动，每个动作都与席卷的沙浪完美融合。逆光中，它的身影与起伏的沙丘勾勒出东方水墨意境。.mp4

进阶用法&部分案例展示
进阶1： 多镜头生成
功能：模型生成前后镜头自然切换，但内容保持一致的视频
提示词 = [场景1] + 镜头切换到 [场景2] 
cases:
暂时无法在飞书文档外展示此内容

进阶2：两个动作生成
功能：模型生成包含两个连贯动作的视频
提示词 = [主体描述]+[动作描述] + [然后、过了一会等连接词] + [动作描述2]
cases：
暂时无法在飞书文档外展示此内容

""".replace("腾讯混元", "").replace("腾讯", "").replace("混元", "")


#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : jina
# @Time         : 2024/7/8 08:44
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :

from meutils.pipe import *
from meutils.schemas.jina_types import BASE_URL, RerankerRequest


@alru_cache()
async def rerank(request: RerankerRequest, api_key: Optional[str] = None):
    # logger.debug("没走缓存")

    payload = request.model_dump()

    api_key = api_key or os.getenv("JINA_API_KEY")
    headers = {
        "authorization": f"Bearer {api_key}"
    }
    async with httpx.AsyncClient(base_url=BASE_URL, headers=headers, timeout=100) as client:
        response = await client.post("/rerank", json=payload)
        # logger.debug(response.text)

        return response.is_success and response.json()


if __name__ == '__main__':
    payload = {
        "model": "jina-reranker-v2-base-multilingual",
        "query": "Organic skincare products for sensitive skin",
        "top_n": 3,
        "documents": [
            "Organic skincare for sensitive skin with aloe vera and chamomile: Imagine the soothing embrace of nature with our organic skincare range, crafted specifically for sensitive skin. Infused with the calming properties of aloe vera and chamomile, each product provides gentle nourishment and protection. Say goodbye to irritation and hello to a glowing, healthy complexion.",
            "New makeup trends focus on bold colors and innovative techniques: Step into the world of cutting-edge beauty with this seasons makeup trends. Bold, vibrant colors and groundbreaking techniques are redefining the art of makeup. From neon eyeliners to holographic highlighters, unleash your creativity and make a statement with every look.",
            "Bio-Hautpflege für empfindliche Haut mit Aloe Vera und Kamille: Erleben Sie die wohltuende Wirkung unserer Bio-Hautpflege, speziell für empfindliche Haut entwickelt. Mit den beruhigenden Eigenschaften von Aloe Vera und Kamille pflegen und schützen unsere Produkte Ihre Haut auf natürliche Weise. Verabschieden Sie sich von Hautirritationen und genießen Sie einen strahlenden Teint.",
            "Neue Make-up-Trends setzen auf kräftige Farben und innovative Techniken: Tauchen Sie ein in die Welt der modernen Schönheit mit den neuesten Make-up-Trends. Kräftige, lebendige Farben und innovative Techniken setzen neue Maßstäbe. Von auffälligen Eyelinern bis hin zu holografischen Highlightern – lassen Sie Ihrer Kreativität freien Lauf und setzen Sie jedes Mal ein Statement.",
            "Cuidado de la piel orgánico para piel sensible con aloe vera y manzanilla: Descubre el poder de la naturaleza con nuestra línea de cuidado de la piel orgánico, diseñada especialmente para pieles sensibles. Enriquecidos con aloe vera y manzanilla, estos productos ofrecen una hidratación y protección suave. Despídete de las irritaciones y saluda a una piel radiante y saludable.",
            "Las nuevas tendencias de maquillaje se centran en colores vivos y técnicas innovadoras: Entra en el fascinante mundo del maquillaje con las tendencias más actuales. Colores vivos y técnicas innovadoras están revolucionando el arte del maquillaje. Desde delineadores neón hasta iluminadores holográficos, desata tu creatividad y destaca en cada look.",
            "针对敏感肌专门设计的天然有机护肤产品：体验由芦荟和洋甘菊提取物带来的自然呵护。我们的护肤产品特别为敏感肌设计，温和滋润，保护您的肌肤不受刺激。让您的肌肤告别不适，迎来健康光彩。",
            "新的化妆趋势注重鲜艳的颜色和创新的技巧：进入化妆艺术的新纪元，本季的化妆趋势以大胆的颜色和创新的技巧为主。无论是霓虹眼线还是全息高光，每一款妆容都能让您脱颖而出，展现独特魅力。",
            "敏感肌のために特別に設計された天然有機スキンケア製品: アロエベラとカモミールのやさしい力で、自然の抱擁を感じてください。敏感肌用に特別に設計された私たちのスキンケア製品は、肌に優しく栄養を与え、保護します。肌トラブルにさようなら、輝く健康な肌にこんにちは。",
            "新しいメイクのトレンドは鮮やかな色と革新的な技術に焦点を当てています: 今シーズンのメイクアップトレンドは、大胆な色彩と革新的な技術に注目しています。ネオンアイライナーからホログラフィックハイライターまで、クリエイティビティを解き放ち、毎回ユニークなルックを演出しましょう。"
        ]
    }

    with timer():
        print(arun(rerank(RerankerRequest(**payload))))

    with timer():
        print(arun(rerank(RerankerRequest(**payload))))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : DeepNN.
# @File         : decorator_utils
# @Time         : 2020/4/30 10:46 上午
# @Author       : yuanjie
# @Email        : meutils@qq.com
# @Software     : PyCharm
# @Description  : 可以看到，当多个装饰器装饰同一个函数时，会是一个嵌套的装饰结果，也就是说，先执行完离函数近的一个装饰器，然后再用离函数远的装饰器来装饰执行结果。
"""
@wrapt.decorator
def noargs(wrapped, instance, args, kwargs):
    logger.info(f'noargs decorator')

    return wrapped(*args, **kwargs)


def withargs(myarg1, myarg2):
    @wrapt.decorator
    def wrapper(wrapped, instance, args, kwargs):
        logger.info(f'withargs decorator: {myarg1}, {myarg2}')
        return wrapped(*args, **kwargs)

    return wrapper
"""
import inspect
import traceback

import wrapt
from collections import OrderedDict
from functools import lru_cache

# ME
from meutils.decorators.common import *
from meutils.decorators.decorator import decorator


class singleton:
    def __init__(self, cls):
        self.cls = cls
        self.instance = None

    def __call__(self, *args, **kwargs):
        if self.instance is None:
            self.instance = self.cls(*args, **kwargs)
        return self.instance


@decorator
def args(wrapped, *args, **kwargs):
    func_name = wrapped.__name__
    logger.debug(f'FUNC-{func_name} ARGS: {args}')
    logger.debug(f'FUNC-{func_name} KWARGS: {kwargs}')
    logger.debug(f'FUNC-{func_name} DEFINED ARGS: {inspect.getfullargspec(wrapped).args}')  # .varargs

    return wrapped(*args, **kwargs)


@decorator
def deprecated(wrapped, *args, **kwargs):
    func_name = wrapped.__name__
    logger.warning(f"{func_name} deprecated")
    return wrapped(*args, **kwargs)


def hdfs_flag(check_dir):  # TODO: zk check
    @wrapt.decorator
    def wrapper(wrapped, instance, args, kwargs):
        from meutils.io import tf_io
        # 不执行
        if tf_io.check_path(f"{check_dir}/_FLAG") or tf_io.check_path(f"{check_dir}/_SUCCESS"):
            logger.info("任务正在执行或者已经执行成功")
            return

        tf_io.make_flag(check_dir)  # 执行中：生成 _FLAG
        _ = wrapped(*args, **kwargs)
        tf_io.rm(f"{check_dir}/_FLAG")  # 执行成功: 删除 _FLAG

        return _

    return wrapper


@decorator  # 更简单
def route_hook(func, *args, **kwargs):
    """
        @app.get("/")
        @route_hook()
        def get(r: Request):
            return r.query_params

    @param func:
    @param args:
    @param kwargs:
    @return:
    """
    output = OrderedDict()
    output['error_code'] = 0
    output['error_msg'] = "SUCCESS"

    try:
        output['data'] = func(*args, **kwargs)

    except Exception as error:
        output['error_code'] = 1  # 通用错误
        output['error_msg'] = traceback.format_exc().strip()  # debug状态获取详细信息


    finally:
        output.update(kwargs)

    return output


if __name__ == '__main__':
    class A:

        def __init__(self, ):
            print('A实例化')


    @singleton
    class B:
        def __init__(self, ):
            print('B实例化')


    for _ in range(3):
        print('\n', _)
        A()
        B()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : siliconflow
# @Time         : 2025/8/15 20:29
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 

from meutils.pipe import *
from meutils.llm.clients import OpenAI

models_mapping = {
    "glm-4.5": "zai-org/GLM-4.5",
    "glm-4.5v": "zai-org/GLM-4.5V",

    "deepseek-v3": "deepseek-ai/DeepSeek-V3",
    "deepseek-v3-0324": "deepseek-ai/DeepSeek-V3",
    "deepseek-v3-250324": "deepseek-ai/DeepSeek-V3",
    "deepseek-chat": "deepseek-ai/DeepSeek-V3",
    "deepseek-v3.1": "deepseek-ai/DeepSeek-V3.1",
    "deepseek-v3-1-250821": "deepseek-ai/DeepSeek-V3.1",

    "qwen3-32b": "Qwen/Qwen3-32B",
    "deepseek-r1": "deepseek-ai/DeepSeek-R1",
    "deepseek-r1-250528": "deepseek-ai/DeepSeek-R1",
    "deepseek-reasoner": "deepseek-ai/DeepSeek-R1",
    "qwen2.5-72b-instruct": "Qwen/Qwen2.5-72B-Instruct-128K",

    "kimi-k2-250711": "moonshotai/Kimi-K2-Instruct",
    "kimi-k2-0711-preview": "moonshotai/Kimi-K2-Instruct",

    "kimi-k2-250905": "moonshotai/Kimi-K2-Instruct-0905",
    "kimi-k2-0905-preview": "moonshotai/Kimi-K2-Instruct-0905",

    "qwen2.5-vl-32b-instruct": "Qwen/Qwen2.5-VL-32B-Instruct",
    "qwen2.5-vl-72b-instruct": "Qwen/Qwen2.5-VL-72B-Instruct",
    "qwen2.5-vl-7b-instruct": "Qwen/Qwen2.5-VL-7B-Instruct",
    "minimax-m1-80k": "MiniMaxAI/MiniMax-M1-80k",
    "qvq-72b-preview": "Qwen/QVQ-72B-Preview",
    "qwen2.5-7b-instruct": "Qwen/Qwen2.5-7B-Instruct",
    "deepseek-r1:1.5b": "deepseek-ai/DeepSeek-R1-Distill-Qwen-1.5B",
    "deepseek-r1-distill-qwen-1.5b": "deepseek-ai/DeepSeek-R1-Distill-Qwen-1.5B",
    "deepseek-r1:7b": "deepseek-ai/DeepSeek-R1-Distill-Qwen-7B",
    "deepseek-r1-distill-qwen-7b": "deepseek-ai/DeepSeek-R1-Distill-Qwen-7B",
    "deepseek-r1:8b": "deepseek-ai/DeepSeek-R1-Distill-Llama-8B",
    "deepseek-r1-distill-llama-8b": "deepseek-ai/DeepSeek-R1-Distill-Llama-8B",
    "qwen2.5-32b-instruct": "Qwen/Qwen2.5-32B-Instruct",

    "flux": "black-forest-labs/FLUX.1-schnell",
    "flux-schnell": "black-forest-labs/FLUX.1-schnell",
    "flux-pro-max": "black-forest-labs/FLUX.1-dev",
    "flux-dev": "black-forest-labs/FLUX.1-dev",
    "flux-pro": "black-forest-labs/FLUX.1-dev",
    "flux.1.1-pro": "black-forest-labs/FLUX.1-dev",
}


def get_models_mapping():
    client = OpenAI(
        api_key=os.getenv("SILICONFLOW_API_KEY"),
        base_url=os.getenv("SILICONFLOW_BASE_URL"),
    )

    models = client.models.list().data
    models = {
        m.id.removeprefix("Pro/").split('/', maxsplit=1)[-1].lower(): m.id.removeprefix("Pro/") for m in models
        if any(i not in m.id.lower() for i in {"stable-diffusion"})
    }
    return {**models, **models_mapping}


if __name__ == '__main__':
    data = get_models_mapping()
    print(bjson(data))
    print(','.join(data))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : MeUtils.
# @File         : shell_utils
# @Time         : 2021/1/19 8:07 下午
# @Author       : yuanjie
# @Email        : meutils@qq.com
# @Software     : PyCharm
# @Description  : 

from meutils.pipe import *


def wget(url, rename=None, is_return=False):
    cmd = f"wget {url}"
    if rename:
        cmd += f" -O {rename}"


if __name__ == '__main__':
    print(Path('xx') / '/x/xx/xxxx')



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : html2image
# @Time         : 2024/9/19 09:03
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 

from meutils.pipe import *


import fal_client

def on_queue_update(update):
    if isinstance(update, fal_client.InProgress):
        for log in update.logs:
           print(log["message"])

result = fal_client.subscribe(
    "fal-ai/flux-pro/v1.1-ultra",
    arguments={
        "prompt": "Extreme close-up of a single tiger eye, direct frontal view. Detailed iris and pupil. Sharp focus on eye texture and color. Natural lighting to capture authentic eye shine and depth. The word \"FLUX\" is painted over it in big, white brush strokes with visible texture."
    },
    with_logs=True,
    on_queue_update=on_queue_update,
)
print(result)


#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : chat
# @Time         : 2025/9/3 14:41
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :  todo


from openai import AsyncOpenAI, OpenAI

from meutils.pipe import *
from meutils.decorators.retry import retrying
from meutils.io.files_utils import to_bytes, guess_mime_type
from meutils.caches import rcache

from meutils.llm.utils import oneturn2multiturn

from meutils.config_utils.lark_utils import get_next_token_for_polling
from meutils.schemas.openai_types import chat_completion, chat_completion_chunk, CompletionRequest, CompletionUsage, \
    ChatCompletion, ChatCompletionChunk
from openai._streaming import AsyncStream, Stream

base_url = "https://longcat.chat/api/v1/chat-completion"
base_url = "https://longcat.chat/api/v1/chat-completion-oversea"
base_url = "https://longcat.chat/api/v1"

cookie = "_lxsdk_cuid=1990e1e8790c8-0b2a66e23040a48-16525636-1fa400-1990e1e8790c8; passport_token_key=AgEGIygg22VuoMYTPonur9FA_-EVg9UXLu3LYOzJ4kIHSjQZeSNhwpytTU_cZFP6V1Juhk0CHMrAgwAAAABYLAAA9vXtnciaZBu2V99EMRJYRHTDSraV_OPLemUuVpi2WLsaa6RqC0PAKAOm6W_hIpbV"


class Completions(object):
    def __init__(self, api_key: Optional[str] = None):
        self.api_key = api_key or cookie

    async def create(self, request: CompletionRequest, **kwargs):
        payload = self.requset2payload(request)
        payload['conversationId'] = await self.create_chat()

        logger.debug(payload)

        headers = {
            'Cookie': self.api_key
        }

        async with httpx.AsyncClient(base_url=base_url, headers=headers, timeout=100) as client:
            async with client.stream("POST", "/chat-completion", json=payload) as response:
                logger.debug(response.status_code)
                response.raise_for_status()

                async for chunk in response.aiter_lines():
                    if chunk := chunk.removeprefix("data:"):
                        _chunk = json.loads(chunk)

                        chat_completion_chunk = ChatCompletionChunk.model_validate(_chunk)
                        # logger.debug(chunk)
                        if request.stream:
                            yield chat_completion_chunk

        if not request.stream:
            # logger.debug(_chunk)
            model = _chunk.get("model", "LongCat-Flash")
            content = _chunk.get("content", "")
            reason_content = _chunk.get("reasonContent", "")
            token_info = _chunk.get("tokenInfo", {})

            chat_completion_chunk.choices[0].content = content
            chat_completion_chunk.choices[0].reason_content = reason_content
            usage = CompletionUsage(
                prompt_tokens=token_info.get("promptTokens"),
                completion_tokens=token_info.get("completionTokens")
            )
            chat_completion.model = model
            chat_completion.usage = usage
            chat_completion.choices = chat_completion_chunk.choices
            # logger.debug(chat_completion)
            yield chat_completion

    def requset2payload(self, request: CompletionRequest):
        payload = {
            "content": oneturn2multiturn(request),  # todo: 多轮
            # "messages": request.messages,
            "reasonEnabled": request.enable_thinking and 1 or 0,
            "searchEnabled": 0,
            "regenerate": 0
        }

        return payload

    async def create_chat(self):
        headers = {
            'Cookie': self.api_key
        }
        payload = {
            "model": "",
            "agentId": ""
        }
        async with httpx.AsyncClient(base_url=base_url, headers=headers, timeout=100) as client:
            response = await client.post("/session-create", json=payload)
            response.raise_for_status()
            # {'code': 0,
            #  'data': {'agent': '1',
            #           'conversationId': 'c1731258-230a-4b2e-b7ef-ea5e83c38e0e',
            #           'createAt': 1756883097539,
            #           'currentMessageId': 0,
            #           'label': '今天',
            #           'model': 'LongCat',
            #           'title': '新对话',
            #           'titleType': 'SYSTEM',
            #           'updateAt': 1756883097539},
            #  'message': 'success'}
            return response.json()['data']['conversationId']


if __name__ == '__main__':
    cookie = "_lxsdk_cuid=1990e1e8790c8-0b2a66e23040a48-16525636-1fa400-1990e1e8790c8; passport_token_key=AgEGIygg22VuoMYTPonur9FA_-EVg9UXLu3LYOzJ4kIHSjQZeSNhwpytTU_cZFP6V1Juhk0CHMrAgwAAAABYLAAA9vXtnciaZBu2V99EMRJYRHTDSraV_OPLemUuVpi2WLsaa6RqC0PAKAOm6W_hIpbV"
    request = CompletionRequest(
        messages=[
            {'role': 'user', 'content': '1+1'},
            {'role': 'assistant', 'content': '2'},
            {'role': 'user', 'content': '裸体傻女'},
        ]
    )
    arun(Completions(api_key=cookie).create(request))
    # arun(Completions(api_key=cookie).create_chat())

    # payload = {
    #     "content": "1+1",
    #     "messages": [
    #         {
    #             "role": "user",
    #             "content": "1+1",
    #             "chatStatus": "FINISHED",
    #             "messageId": 0,
    #             "idType": "system"
    #         }
    #     ],
    #     "reasonEnabled": 0,
    #     "searchEnabled": 0,
    #     "regenerate": 0
    # }
    #
    # Completions(api_key=cookie).stream_create(payload)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : wechat_types
# @Time         : 2024/8/8 18:42
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 


from meutils.pipe import *
from pydantic import constr


# {"type": "text",  "content": 'xx'}
# {"type": "error", "content": 'xx'}

# {"type": "image", "content": 'https://oss.ffire.cc/files/xx.png'}

# {"type": "audio", "content": 'https://oss.ffire.cc/files/xx.mp3'}
# {"type": "file", "content": 'https://oss.ffire.cc/files/xx.mp3'}

# {"type": "video", "content": 'https://oss.ffire.cc/files/vidu.mp4'}
class HookResponse(BaseModel):
    type: str = 'text'
    content: Optional[str] = ''


class HookResponses(BaseModel):
    responses: List[HookResponse]


class RecommendInfo(BaseModel):
    UserName: str = ''
    NickName: str = ''
    QQNum: int = '0'
    Province: str = ''
    City: str = ''
    Content: str = ''
    Signature: str = ''
    Alias: str = ''
    Scene: int = '0'
    VerifyFlag: int = '0'
    AttrStatus: int = '0'
    Sex: int = '0'
    Ticket: str = ''
    OpCode: int = '0'


class Member(BaseModel):
    MemberList: list = []
    Uin: int = '0'
    UserName: str = ''
    NickName: str = ''
    AttrStatus: int = 0
    PYInitial: str = ''
    PYQuanPin: str = ''
    RemarkPYInitial: str = ''
    RemarkPYQuanPin: str = ''
    MemberStatus: int = 0
    DisplayName: str = ''
    KeyWord: str = ''


# class Self(Member):
#     """机器人属性"""
#     pass


class User(BaseModel):
    """群属性"""
    MemberList: List[Member] = None

    Uin: int = '0'
    UserName: str = '@@...'
    NickName: constr(to_lower=True) = ''  # 群昵称
    HeadImgUrl: str = '/cgi-bin/mmwebwx-bin/webwxgetheadimg?seq=805036502&username=@@...&skey='
    ContactFlag: int = 0
    MemberCount: int = 0
    RemarkName: str = ''
    HideInputBarFlag: int = 0
    Sex: int = 0
    Signature: str = ''
    VerifyFlag: int = 0
    OwnerUin: int = 0
    PYInitial: str = ''  # 'XCHATCHATLLM'
    PYQuanPin: str = ''  # 'Xchatchatllm'
    RemarkPYInitial: str = ''
    RemarkPYQuanPin: str = ''
    StarFriend: int = 0
    AppAccountFlag: int = 0
    Statues: int = 1
    AttrStatus: int = 0
    Province: str = ''
    City: str = ''
    Alias: str = ''
    SnsFlag: int = 0
    UniFriend: int = 0
    DisplayName: str = ''
    ChatRoomId: int = 0
    KeyWord: str = ''
    EncryChatRoomId: str = '@...'
    IsOwner: int = '1'
    IsAdmin: Optional[str] = None

    Self: Any = Member()  # 机器人微信信息 # Self


class Message(BaseModel):
    MsgId: str = ''
    FromUserName: str = '@...'
    ToUserName: str = '@@...'  # @@是接收信息的群ID
    MsgType: int = 49
    Content: str = ''
    Status: int = 3
    ImgStatus: int = 1
    CreateTime: int = Field(default_factory=lambda: int(time.time()))
    VoiceLength: int = 0
    PlayLength: int = 0
    FileName: str = ''  # 'xx.doc'
    FileSize: str = '633344'
    MediaId: str = '@crypt_...'
    Url: str = ''
    AppMsgType: int = 6
    StatusNotifyCode: int = 0
    StatusNotifyUserName: str = ''

    RecommendInfo: Any = RecommendInfo()

    ForwardFlag: int = '0'
    AppInfo: dict = {'AppID': '', 'Type': 0}
    HasProductId: int = '0'
    Ticket: str = ''
    ImgHeight: int = '0'
    ImgWidth: int = '0'
    SubMsgType: int = '0'
    NewMsgId: int = '1316678251262129000'
    OriContent: str = ''
    EncryFileName: str = '%E8%80%B6%E8%B7%AF%E6%92%92%E5%86%B7%E4%B8%89%E5%8D%83%E5%B9%B4%2Edoc'
    ActualNickName: str = ''  # 群里展示的昵称

    IsAt: bool = False
    ActualUserName: str = '@...'

    # user: User = Field(alias='User')
    User: User

    Type: str = 'Attachment'
    Text: Any = '{}'  # 可能是函数

    # 增强
    chatroom_name: str = None
    chatroom_id: str = None

    user_name: str = None

    # download: Callable = None

    def __init__(self, **data: Any):
        super().__init__(**data)

        self.chatroom_name = self.User.NickName  # 已被转小写
        self.chatroom_id = self.User.UserName

        self.user_name = self.User.NickName  # 人昵称 不是机器人昵称


@dataclass(init=False)
class MessageType(object):
    """
    图片或表情（PICTURE）、录制（RECORDING）、附件（ATTACHMENT）、小视频（VIDEO）、文本（TEXT），地图（MAP），名片（CARD），
    通知（NOTE），好友邀请（FRIENDS）、语音（RECORDING）、系统消息（SYSTEM）
    分享（SHARING）:
        小程序     'AppMsgType': 33
        小视频     'AppMsgType': 51 'Type': 'Sharing', 'Text': '当前微信版本不支持展示该内容，请升级至最新版本。'
        微信看一看 ’AppMsgType': 5   'Type': 'Sharing', 'Text': '加沙一难民营被以色列“完全摧毁”' 'FileName': '加沙一难民营被以色列“完全摧毁”'
        公众号文章 'AppMsgType': 5   'Type': 'Sharing', 'Text': 'Skywork-13B：昆仑万维集团-天工团队彻底开源代码和数据集' 'FileName': 'Skywork-13B：昆仑万维集团-天工团队彻底开源代码和数据集'
    """
    TEXT = 'Text'
    MAP = 'Map'
    CARD = 'Card'
    NOTE = 'Note'
    SHARING = 'Sharing'
    PICTURE = 'Picture'
    RECORDING = 'Recording'  #
    VOICE = 'Recording'  #
    ATTACHMENT = 'Attachment'
    VIDEO = 'Video'
    FRIENDS = 'Friends'
    SYSTEM = 'System'

    INCOME_MSG = [TEXT, MAP, CARD, NOTE, SHARING, PICTURE, RECORDING, VOICE, ATTACHMENT, VIDEO, FRIENDS, SYSTEM]


# 朋友信息
{
    "MemberList": "<ContactList: []>",
    "Uin": 0,
    "UserName": "@aa8e769e75daa860fd7bebf0a7c3af32bc4b731f920ef02a3830328ce936c492",
    "NickName": "firebot",
    "HeadImgUrl": "/cgi-bin/mmwebwx-bin/webwxgeticon?seq=766248947&username=@aa8e769e75daa860fd7bebf0a7c3af32bc4b731f920ef02a3830328ce936c492&skey=@crypt_9ee03a29_289538cfb03502fc9947971cb5f890d5",
    "ContactFlag": 2051,
    "MemberCount": 0,
    "RemarkName": "",
    "HideInputBarFlag": 0,
    "Sex": 1,
    "Signature": "搞钱<span class=\"emoji emoji1f4b0\"></span>",
    "VerifyFlag": 0,
    "OwnerUin": 0,
    "PYInitial": "FIREBOT",
    "PYQuanPin": "firebot",
    "RemarkPYInitial": "",
    "RemarkPYQuanPin": "",
    "StarFriend": 0,
    "AppAccountFlag": 0,
    "Statues": 0,
    "AttrStatus": 102725,
    "Province": "江苏",
    "City": "苏州",
    "Alias": "",
    "SnsFlag": 257,
    "UniFriend": 0,
    "DisplayName": "",
    "ChatRoomId": 0,
    "KeyWord": "",
    "EncryChatRoomId": "",
    "IsOwner": 0
}

# USer
{
    "MemberList": "<ContactList: []>",
    "Uin": 0,
    "UserName": "@aa8e769e75daa860fd7bebf0a7c3af32bc4b731f920ef02a3830328ce936c492",
    "NickName": "firebot",
    "HeadImgUrl": "/cgi-bin/mmwebwx-bin/webwxgeticon?seq=766248947&username=@aa8e769e75daa860fd7bebf0a7c3af32bc4b731f920ef02a3830328ce936c492&skey=@crypt_9ee03a29_289538cfb03502fc9947971cb5f890d5",
    "ContactFlag": 2051,
    "MemberCount": 0,
    "RemarkName": "",
    "HideInputBarFlag": 0,
    "Sex": 1,
    "Signature": "搞钱<span class=\"emoji emoji1f4b0\"></span>",
    "VerifyFlag": 0,
    "OwnerUin": 0,
    "PYInitial": "FIREBOT",
    "PYQuanPin": "firebot",
    "RemarkPYInitial": "",
    "RemarkPYQuanPin": "",
    "StarFriend": 0,
    "AppAccountFlag": 0,
    "Statues": 0,
    "AttrStatus": 102725,
    "Province": "江苏",
    "City": "苏州",
    "Alias": "",
    "SnsFlag": 257,
    "UniFriend": 0,
    "DisplayName": "",
    "ChatRoomId": 0,
    "KeyWord": "",
    "EncryChatRoomId": "",
    "IsOwner": 0
}

messagetype_map = {

    "[文件]": MessageType.ATTACHMENT,  # 语音也算
    "[链接]": MessageType.SHARING,

    # 无法绝对定位，只能相对【根据最新图片或者视频】
    "[视频]": MessageType.VIDEO,
    "[图片]": MessageType.PICTURE,

}


class QuoteMessage(object):
    """
    「firebot：qqqqqqq」
    「NickName：[文件]hello.mp3」
    「firebot：[文件]耶路撒冷三千年.doc」
    「Betterme：[链接]国产AI火爆海外，7个月收入近千万」

    「Betterme：[图片]」
    「Betterme：[视频]」
    - - - - - - - - - - - - - - -
    这是个问题

    @firebot 这是个问题 # todo


    """

    def __init__(self, text_msg: str):
        q1, q2 = text_msg.split('\n- - - - - - - - - - - - - - -\n')  # ['「Betterme：[链接]国产AI火爆海外，7个月收入近千万」', '这是个问题']
        self.text_msg = text_msg.strip()
        self.nickname, quote = q1.strip()[1:-1].split('：', maxsplit=1)
        self.context_type = messagetype_map.get(quote[:4], MessageType.TEXT)  # [链接]国产AI火爆海外，7个月收入近千万
        self.context = quote[4:] if self.context_type != MessageType.TEXT else quote  # 上下文

        self.question = q2.strip()


# python#获取公众号名称为“Python之禅”的信息
# mp_username = itchat.search_mps(name='Python之禅')[0]['UserName']
# mp_info = itchat.update_friend(username=mp_username, detailed=True)


if __name__ == '__main__':
    # msg = """
    # 「Betterme：东北证券AI架构.pptx」\n- - - - - - - - - - - - - - -\n@Betterme\u2005/tts不知道
    # """.strip()
    # rprint(QuoteMessage(msg).__dict__)
    # rprint(QuoteMessage(msg).question.split('/tts', 1)[-1])
    User



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : llm
# @Time         : 2025/6/4 13:33
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 

from meutils.pipe import *
from meutils.schemas.openai_types import CompletionRequest
from meutils.llm.utils import oneturn2multiturn

from fal_client.client import AsyncClient, SyncClient, Status, FalClientError

models = "anthropic/claude-3.7-sonnet,anthropic/claude-3.5-sonnet, anthropic/claude-3-5-haiku, anthropic/claude-3-haiku, google/gemini-pro-1.5, google/gemini-flash-1.5, google/gemini-flash-1.5-8b, meta-llama/llama-3.2-1b-instruct, meta-llama/llama-3.2-3b-instruct, meta-llama/llama-3.1-8b-instruct, meta-llama/llama-3.1-70b-instruct, openai/gpt-4o-mini, openai/gpt-4o, deepseek/deepseek-r1"

async def create(
        request: CompletionRequest,
        api_key: Optional[str] = None,
):
    """
    Prompt length must be less than 5000 characters

    :param request:
    :param api_key:
    :return:
    """
    if (not request.stream and request.max_tokens and request.max_tokens < 11):
        yield " "
        return

    api_key = api_key and api_key.removeprefix("fal-")

    prompt = oneturn2multiturn(request.messages)

    arguments = {
        "model": request.model,
        "system_prompt": request.system_instruction,
        "prompt": prompt,
        "temperature": request.temperature,
        "reasoning": request.reasoning_effort is not None,
        "max_tokens": 1,
    }
    client = AsyncClient(key=api_key)

    if request.stream:
        yield ""

        stream = client.stream("fal-ai/any-llm", arguments=arguments)
        prefix = ""
        i = 0
        async for chunk in stream:
            _ = chunk.get("output")
            yield _.removeprefix(prefix)
            prefix = _
            i += 1
            if request.max_tokens and i > request.max_tokens:
                break

    else:
        response = await client.run("fal-ai/any-llm", arguments=arguments)
        # {'error': None, 'output': '1 + 1 = 2', 'partial': False, 'reasoning': None}
        if response.get("error"):
            logger.error(response)
        yield response.get("output")


if __name__ == '__main__':
    request = CompletionRequest(
        model="anthropic/claude-3.7-sonnet",
        messages=[
            # {"role": "user", "content": "1+1"},
            {"role": "user",
             "content": [{
                 "type": "text",
                 "text": "1+1"*10000
             }]
             },

        ],
        stream=True,
    )
    arun(create(request))



# -*- coding: utf-8 -*-

from __future__ import absolute_import

from . import archive
from . import audio
from . import application
from . import document
from . import font
from . import image
from . import video
from .base import Type  # noqa

# Supported image types
IMAGE = (
    image.Dwg(),
    image.Xcf(),
    image.Jpeg(),
    image.Jpx(),
    image.Apng(),
    image.Png(),
    image.Gif(),
    image.Webp(),
    image.Tiff(),
    image.Cr2(),
    image.Bmp(),
    image.Jxr(),
    image.Psd(),
    image.Ico(),
    image.Heic(),
    image.Dcm(),
    image.Avif(),
)

# Supported video types
VIDEO = (
    video.M3gp(),
    video.Mp4(),
    video.M4v(),
    video.Mkv(),
    video.Mov(),
    video.Avi(),
    video.Wmv(),
    video.Mpeg(),
    video.Webm(),
    video.Flv(),
)

# Supported audio types
AUDIO = (
    audio.Aac(),
    audio.Midi(),
    audio.Mp3(),
    audio.M4a(),
    audio.Ogg(),
    audio.Flac(),
    audio.Wav(),
    audio.Amr(),
    audio.Aiff(),
)

# Supported font types
FONT = (font.Woff(), font.Woff2(), font.Ttf(), font.Otf())

# Supported archive container types
ARCHIVE = (
    archive.Br(),
    archive.Rpm(),
    archive.Dcm(),
    archive.Epub(),
    archive.Zip(),
    archive.Tar(),
    archive.Rar(),
    archive.Gz(),
    archive.Bz2(),
    archive.SevenZ(),
    archive.Pdf(),
    archive.Exe(),
    archive.Swf(),
    archive.Rtf(),
    archive.Nes(),
    archive.Crx(),
    archive.Cab(),
    archive.Eot(),
    archive.Ps(),
    archive.Xz(),
    archive.Sqlite(),
    archive.Deb(),
    archive.Ar(),
    archive.Z(),
    archive.Lzop(),
    archive.Lz(),
    archive.Elf(),
    archive.Lz4(),
    archive.Zstd(),
)

# Supported archive container types
APPLICATION = (
    application.Wasm(),
)

# Supported document types
DOCUMENT = (
    document.Doc(),
    document.Docx(),
    document.Odt(),
    document.Xls(),
    document.Xlsx(),
    document.Ods(),
    document.Ppt(),
    document.Pptx(),
    document.Odp(),
)


# Expose supported type matchers
TYPES = list(IMAGE + AUDIO + VIDEO + FONT + DOCUMENT + ARCHIVE + APPLICATION)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : qwen
# @Time         : 2025/1/17 16:45
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :
"""
 File "/usr/local/lib/python3.10/site-packages/meutils/llm/completions/qwenllm.py", line 47, in create
    yield response.choices[0].message.content
AttributeError: 'str' object has no attribute 'choices'

"""
import time

from openai import AsyncOpenAI

from meutils.pipe import *
from meutils.decorators.retry import retrying
# from meutils.oss.ali_oss import qwenai_upload
from meutils.io.files_utils import to_bytes, guess_mime_type
from meutils.caches import rcache

from meutils.llm.openai_utils import to_openai_params

from meutils.config_utils.lark_utils import get_next_token_for_polling
from meutils.schemas.openai_types import chat_completion, chat_completion_chunk, CompletionRequest, CompletionUsage, \
    ChatCompletion

FEISHU_URL = "https://xchatllm.feishu.cn/sheets/Bmjtst2f6hfMqFttbhLcdfRJnNf?sheet=PP1PGr"

base_url = "https://chat.qwen.ai/api"
DEFAUL_MODEL = "qwen3-235b-a22b"

from fake_useragent import UserAgent

ua = UserAgent()

thinking_budget_mapping = {
    "low": 1000,
    "medium": 8000,
    "high": 24000
}

COOKIE = """
_gcl_au=1.1.1093269050.1756349377;xlly_s=1;_bl_uid=LXmp28z7dwezpmyejeXL9wh6U1Rb;cnaui=310cbdaf-3754-461c-a3ff-9ec8005329c9;isg=BCQknTSrEFaWtGtm_x0nSvW89SQWvUgnEt_awT5Fz--w6cezYs52t7gDqUFxMYB_;ssxmod_itna2=1-iqGhD50IThkG8Dhx_xmuxWKUt_EoG7DzxC5KY0CDmxjKidDRDB40QRTnf_Ti=qaeGrMwxrDPxD3r5iY80q7DFg0WeDBk4uAn8mY3vKFTl7S9o7EaoSeXnAOPBSok57ccC4rhgutjg2_8D7jk_lChKSSdfMbm2lKAUlRwIqjSxeNld4tMGxFev6zkUel_6LR_foIGQa5L4PuCGa6dUqvwISCODQh2TC6wQ1Hu=Ll=W4=W6s1E_V8Dr_1gDECghksL8zvQHiPI60ChnPodSvFnHjE2iXzGDdE_I5876eQ03cEzaFsA48KQLReNjiPp1I0EfN=5a=dziPIxXxcpup5zmGM2L48PYjAqiUxrwVQDY4vN=Ni_pu6pxOYEBNL7YA6RPcRy7Ak=Y5PPbZRAi242ulCDx3oZCGH2YE6p3lD4gPIOKMWX6AbmhYLrcMeXcYdjzGgaYEDq4DUtKP/1jf=vXt=MQXoZ23BWlE5h06cjceY_Bxw3AH3KeBaxT4pHEt19QSlaO20G9DfDq7Wf3BvV=5X/BYd54Y44nUHOfH_fV2mHKNz0W4lxjWDjY9H3m00I3cOIf3C6q7Y_CNx1sG1nwsiDUA34QDtsC8jw1YD;sca=aefac4ee;acw_tc=0a03e54a17574902359765651e1e9cf9780667ec2a2cefacce63a0b954bb63;atpsida=7e2e4dcfcd1c4a4530dd3395_1757490275_2;aui=310cbdaf-3754-461c-a3ff-9ec8005329c9;cna=KP9DIEqqyjUCATrw/+LjJV8F;ssxmod_itna=1-iqGhD50IThkG8Dhx_xmuxWKUt_EoG7DzxC5KY0CDmxjKidDRDB40QRTnf_Ti=qaeGrMwxrDyxGXoNexiNDAg40iDCbmLx5Yjdrq4NFtrojKaIjL4Q43rj9_8m0tnY/qmUTMU6Rljs7s66tqGI_DG2DYoDCqDS0DD99OdD4_3Dt4DIDAYDDxDWIeDB4ZrQDGPtLQut5eqKwBrtxi3QWWPQDiUCxivW56Wt5C_GwDDliPGfWepi20iDbqDuzKqaBeDLRPpvxB=PxYPmjmkUxBQGHniO/WWb2BkrGOGBKim6mTeM0O_qBGGhfyDGxNI0eYT44YxS4VQYQjytGBFDDWgL4_AzKl_TYx_7CIq13b1_BDCgtIQi_GK=DKoBddAQ2wmlBx/DYfjGeAzNA4aDhLlxtlzaiDD;tfstk=geyx-Dg4-gKxmZY2DPflIb1e9VjuX_qVPrrBIV0D1zU85rs2IlGD5dU-rirmhZz8VyUp5diXCbw8-cTbnog6NdUtyi8XG1y-XPUwhK1V7utSbmnbWlSqfVa_WVVcZ9q40Ak1-lBhKo8RmR1gkd9XVGitjCsoG1PJiYM1-wXoYFbWuAaGVPsqNugrXctjCPOWFD0ScEOb54GSADYjCAa12YiZcKi6GfZ5Fm0S5AM_5us-j4ijCAa_VgnZs7QtPngMBSZPyLNC_OpMI8nxMVpikdIljmcxRogRRwwJ3j3Qc4p18hgjf4iLFN8m78E7hD45EKHTVcUKh7QWHPEbTPoQlTLjDSZLxbeNRdnzNrloD7I6NqZIlrhY9Usx7-4YBjyC-Lubw8ytn7bvKVl_Z-lUna9xdWrmn5a5fdgTNcsPDJ2LcuTnJcAf2gdw_jirJjsq62sn5Go--iB9_ClP403h2wOw_jir22jA-CRZag5..;token=eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9.eyJpZCI6IjMxMGNiZGFmLTM3NTQtNDYxYy1hM2ZmLTllYzgwMDUzMjljOSIsImxhc3RfcGFzc3dvcmRfY2hhbmdlIjoxNzUwNjYwODczLCJleHAiOjE3NTgwOTUwMzd9.JdVvyPkln2HcGm6ib0FKaF1qQ87lG1nf70oezhYZ2Jg;x-ap=cn-hongkong
""".strip()


@retrying()
async def to_file(file, api_key, cookie: Optional[str] = None):
    qwen_client = AsyncOpenAI(
        base_url="https://all.chatfire.cn/qwen/v1",
        api_key=api_key,
        default_headers={
            'User-Agent': ua.random,
            'Cookie': cookie or COOKIE
        }
    )
    filename = Path(file).name if isinstance(file, str) else 'untitled'
    mime_type = guess_mime_type(file)
    file_bytes: bytes = await to_bytes(file)
    file = (filename, file_bytes, mime_type)
    file_object = await qwen_client.files.create(file=file, purpose="file-extract")
    logger.debug(file_object)
    return file_object


async def create(request: CompletionRequest, token: Optional[str] = None, cookie: Optional[str] = None):
    cookie = cookie or COOKIE

    if request.temperature > 1:
        request.temperature = 1

    token = token or await get_next_token_for_polling(feishu_url=FEISHU_URL, from_redis=True)

    logger.debug(token)

    default_query = None

    client = AsyncOpenAI(
        base_url=base_url,
        api_key=token,
        default_headers={
            'User-Agent': ua.random,
            'Cookie': cookie,
        },

        default_query=default_query
    )
    # qwen结构
    model = request.model.lower()
    if any(i in model for i in ("research",)):  # 遇到错误 任意切换
        request.model = DEFAUL_MODEL
        request.messages[-1]['chat_type'] = "deep_research"


    elif any(i in model for i in ("search",)):
        request.model = DEFAUL_MODEL
        request.messages[-1]['chat_type'] = "search"

    # 混合推理
    if (request.reasoning_effort
            or request.last_user_content.startswith("/think")
            or request.enable_thinking
            or hasattr(request, "thinking_budget")
            or any(i in model for i in ("qwq", "qvq", "think", "thinking"))
    ):
        if "qwen-plus-2025-09-11" in model:
            request.model = "qwen-plus-2025-09-11"
        else:
            request.model = DEFAUL_MODEL

        feature_config = {"thinking_enabled": True, "output_schema": "phase"}
        feature_config["thinking_budget"] = thinking_budget_mapping.get(request.reasoning_effort, 1024)
        request.messages[-1]['feature_config'] = feature_config

    # 适配thinking

    if "omni" in model:
        request.max_tokens = 2048

    # 多模态: todo
    # if any(i in request.model.lower() for i in ("-vl", "qvq")):
    #     # await to_file
    last_message = request.messages[-1]
    logger.debug(last_message)

    if last_message.get("role") == "user":
        user_content = last_message.get("content")
        if isinstance(user_content, list):
            for i, content in enumerate(user_content):
                if content.get("type") == 'file_url':  # image_url file_url video_url
                    url = content.get(content.get("type")).get("url")
                    file_object = await to_file(url, token, cookie)

                    user_content[i] = {"type": "file", "file": file_object.id}

                elif content.get("type") == 'image_url':
                    url = content.get(content.get("type")).get("url")
                    file_object = await to_file(url, token, cookie)

                    user_content[i] = {"type": "image", "image": file_object.id}

                elif content.get("type") == 'input_audio':
                    url = content.get(content.get("type")).get("data")
                    file_object = await to_file(url, token, cookie)

                    user_content[i] = {"type": "image", "image": file_object.id}

        elif user_content.startswith("http"):
            file_url, user_content = user_content.split(maxsplit=1)

            user_content = [{"type": "text", "text": user_content}]

            file_object = await to_file(file_url, token, cookie)

            content_type = file_object.meta.get("content_type", "")
            if content_type.startswith("image"):
                user_content.append({"type": "image", "image": file_object.id})
            else:
                user_content.append({"type": "file", "file": file_object.id})

        request.messages[-1]['content'] = user_content

    logger.debug(request)

    request.incremental_output = True  # 增量输出
    data = to_openai_params(request)

    logger.debug(data)

    # 流式转非流
    data['stream'] = True
    chunks = await client.chat.completions.create(**data)

    idx = 0
    nostream_content = ""
    nostream_reasoning_content = ""
    chunk = None
    usage = None
    async for chunk in chunks:
        # logger.debug(chunk)
        if not chunk.choices: continue

        content = chunk.choices[0].delta.content or ""
        if hasattr(chunk.choices[0].delta, "phase") and chunk.choices[0].delta.phase == "think":
            chunk.choices[0].delta.content = ""
            chunk.choices[0].delta.reasoning_content = content
            nostream_reasoning_content += content

        # logger.debug(chunk.choices[0].delta.content)
        nostream_content += chunk.choices[0].delta.content
        usage = chunk.usage or usage

        if request.stream:
            yield chunk

        idx += 1
        if idx == request.max_tokens:
            break

    if not request.stream:
        logger.debug(chunk)
        if hasattr(usage, "output_tokens_details"):
            usage.completion_tokens_details = usage.output_tokens_details
        if hasattr(usage, "input_tokens"):
            usage.prompt_tokens = usage.input_tokens
        if hasattr(usage, "output_tokens"):
            usage.completion_tokens = usage.output_tokens

        chat_completion.usage = usage
        chat_completion.choices[0].message.content = nostream_content
        chat_completion.choices[0].message.reasoning_content = nostream_reasoning_content

        yield chat_completion


if __name__ == '__main__':
    # [
    #     "qwen-plus-latest",
    #     "qvq-72b-preview",
    #     "qwq-32b-preview",
    #     "qwen2.5-coder-32b-instruct",
    #     "qwen-vl-max-latest",
    #     "qwen-turbo-latest",
    #     "qwen2.5-72b-instruct",
    #     "qwen2.5-32b-instruct"
    # ]

    user_content = [
        {
            "type": "text",
            "text": "主体文字'诸事皆顺'，超粗笔画、流畅飘逸、有飞白效果的狂野奔放草书字体，鎏金质感且有熔金流动感和泼溅金箔效果，黑色带细微噪点肌理背景，英文'GOOD LUCK'浅金色或灰白色，有淡淡的道家符文点缀,书法字体海报场景，传统书法与现代设计融合风格,特写,神秘奢华充满能量,焦点清晰，对比强烈"
        },
        # {
        #     "type": "image_url",
        #     "image_url": {
        #         "url": "https://fyb-pc-static.cdn.bcebos.com/static/asset/homepage@2x_daaf4f0f6cf971ed6d9329b30afdf438.png"
        #     }
        # }
    ]

    user_content = "主体文字'诸事皆顺'，超粗笔画、流畅飘逸、有飞白效果的狂野奔放草书字体，鎏金质感且有熔金流动感和泼溅金箔效果，黑色带细微噪点肌理背景，英文'GOOD LUCK'浅金色或灰白色，有淡淡的道家符文点缀,书法字体海报场景，传统书法与现代设计融合风格,特写,神秘奢华充满能量,焦点清晰，对比强烈"
    # {
    #     "type": "image_url",
    #     "image_url": {
    #         "url": "https://fyb-pc-static.cdn.bcebos.com/static/asset/homepage@2x_daaf4f0f6cf971ed6d9329b30afdf438.png"
    #     }
    # }

    # user_content = "1+1"
    # user_content = "/think 1+1"

    # user_content = [
    #     {
    #         "type": "text",
    #         "text": "总结下"
    #     },
    #     {
    #         "type": "file_url",
    #         "file_url": {
    #             "url": "https://oss.ffire.cc/files/AIGC.pdf"
    #         }
    #     }
    #
    # ]

    # user_content = [
    #     {
    #         "role": "user",
    #         "content": [
    #             {
    #                 "type": "input_audio",
    #                 "input_audio": {
    #                     "data": "https://help-static-aliyun-doc.aliyuncs.com/file-manage-files/zh-CN/20250211/tixcef/cherry.wav",
    #                     "format": "wav",
    #                 },
    #             },
    #             {"type": "text", "text": "这段音频在说什么"},
    #         ],
    #     },
    # ]

    request = CompletionRequest(
        # model="qwen3-235b-a22b",
        # model="qwen3-235b-a22b-thinking-2507",
        # model="qwen3-max-preview",
        # model="qwen-plus-2025-09-11", # qwen3-next-80b-a3b
        model="qwen-plus-2025-09-11-thinking",  # qwen3-next-80b-a3b

        # model="qwen-turbo-2024-11-01",
        # model="qwen-max-latest",
        # model="qvq-max-2025-03-25",
        # model="qvq-72b-preview-0310",
        # model="qwen2.5-omni-7b",
        # model="qwen-image",
        # model="qwen-plus",

        # model="qwen-max-latest-search",
        # model="qwq-max",
        # model="qwq-32b-preview",
        # model="qwq-max-search",

        # model="qwen2.5-vl-72b-instruct",

        # model="qwen-plus-latest",
        # model="qwen3-235b-a22b",
        # model="qwen3-30b-a3b",
        # model="qwen3-32b",

        # model="qwen-omni-turbo-0119",

        # max_tokens=1,
        # max_tokens=100,

        messages=[
            {
                'role': 'user',
                'content': '1+1',
            },
            {
                'role': 'assistant',
                'content': '3',
            },
            {
                'role': 'user',
                # 'content': '今天南京天气',
                # 'content': "9.8 9.11哪个大",
                # 'content': 'https://oss.ffire.cc/files/AIGC.pdf 总结下',
                # 'content': ' 总结下',

                # "chat_type": "search", deep_research

                # 'content': user_content,
                'content': "错了",

                # "content": [
                #     {
                #         "type": "text",
                #         "text": "总结下",
                #         "chat_type": "t2t",
                #         "feature_config": {
                #             "thinking_enabled": False
                #         }
                #     },
                #     {
                #         "type": "file",
                #         "file": "2d677df1-45b2-4f30-829f-0d42b2b07136"
                #     }
                # ]

                # "content": [
                #     {
                #         "type": "text",
                #         "text": "总结下",
                #         "chat_type": "t2t",
                #         "feature_config": {
                #             "thinking_enabled": False
                #         }
                #     },
                #     {
                #         "type": "file_url",
                #         "file_url": {
                #           "url": 'xxxxxxx'
                #         }
                #     }
                # ]
                # "content": [
                #     {
                #         "type": "text",
                #         "text": "总结下",
                #         # "chat_type": "t2t"
                #
                #     },
                # {
                #     "type": "image",
                #     "image": "703dabac-b0d9-4357-8a85-75b9456df1dd"
                # },
                # {
                #     "type": "image",
                #     "image": "https://oss.ffire.cc/files/kling_watermark.png"
                #
                # }
                # ]

            },

        ],
        stream=False,

        # reasoning_effort="low",
        # enable_thinking=True,
        # thinking_budget=1024,
        # stream_options={"include_usage": True},

    )
    token = None

    token = "eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9.eyJpZCI6IjMxMGNiZGFmLTM3NTQtNDYxYy1hM2ZmLTllYzgwMDUzMjljOSIsImxhc3RfcGFzc3dvcmRfY2hhbmdlIjoxNzUwNjYwODczLCJleHAiOjE3NTgwOTUwMzd9.JdVvyPkln2HcGm6ib0FKaF1qQ87lG1nf70oezhYZ2Jg"

    arun(create(request, token))

    # arun(to_file("https://help-static-aliyun-doc.aliyuncs.com/file-manage-files/zh-CN/20250211/tixcef/cherry.wav", token))

    # arun(create_new_chat(token))



#!/usr/bin/env python3
from typing import Dict

from nicegui import ui

tab_names = ['A', 'B', 'C']

# necessary until we improve native support for tabs (https://github.com/zauberzeug/nicegui/issues/251)


def switch_tab(msg: Dict) -> None:
    name = msg['args']
    tabs.props(f'model-value={name}')
    panels.props(f'model-value={name}')


with ui.header().classes(replace='row items-center') as header:
    ui.button(on_click=lambda: left_drawer.toggle()).props('flat color=white icon=menu')
    with ui.element('q-tabs').on('update:model-value', switch_tab) as tabs:
        for name in tab_names:
            ui.element('q-tab').props(f'name={name} label={name}')

with ui.footer(value=False) as footer:
    ui.label('Footer')

with ui.left_drawer().classes('bg-blue-100') as left_drawer:
    ui.label('Side menu')

with ui.page_sticky(position='bottom-right', x_offset=20, y_offset=20):
    ui.button(on_click=footer.toggle).props('fab icon=contact_support')


# the page content consists of multiple tab panels
with ui.element('q-tab-panels').props('model-value=A animated').classes('w-full') as panels:
    for name in tab_names:
        with ui.element('q-tab-panel').props(f'name={name}').classes('w-full'):
            ui.label(f'Content of {name}')

ui.run()


#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : utils
# @Time         : 2024/6/20 09:08
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :


from meutils.pipe import *
from meutils.schemas.openai_types import CompletionRequest



def oneturn2multiturn(messages: Union[dict, CompletionRequest], template: Optional[str] = None, ignore_system: bool = True):
    """todo: https://github.com/hiyouga/LLaMA-Factory/blob/e898fabbe3efcd8b44d0e119e7afaed4542a9f39/src/llmtuner/data/template.py#L423-L427

    _register_template(
    name="qwen",
    format_user=StringFormatter(slots=["<|im_start|>user\n{{content}}<|im_end|>\n<|im_start|>assistant\n"]),
    format_system=StringFormatter(slots=["<|im_start|>system\n{{content}}<|im_end|>\n"]),
    format_observation=StringFormatter(slots=["<|im_start|>tool\n{{content}}<|im_end|>\n<|im_start|>assistant\n"]),
    format_separator=EmptyFormatter(slots=["\n"]),
    default_system="You are a helpful assistant.",
    stop_words=["<|im_end|>"],
    replace_eos=True,
)
    :return:
    """
    # from jinja2 import Template, Environment, PackageLoader, FileSystemLoader
    #
    # system_template = Template("<|im_start|>system\n{{content}}<|im_end|>\n")  # .render(content='xxxx')
    # user_template = Template("<|im_start|>user\n{{content}}<|im_end|>\n")  # 最后<|im_start|>assistant\n
    # assistant_template = Template("<|im_start|>assistant\n{{content}}<|im_end|>\n")

    # todo: [{"type": "image_url", "image_url": {"url": ""}}]] 单独处理
    # 混元不是很感冒
    # context = "\n"
    # for message in messages:
    #     role, content = message.get("role"), message.get("content")
    #     context += f"<|im_start|>{role}\n{content}<|im_end|>\n"
    # context += "<|im_start|>assistant\n"
    if isinstance(messages, CompletionRequest):
        messages = messages.messages


    if len(messages) == 1:
        content = messages[0].get("content")
        if isinstance(content, list):
            content = content[-1].get('text', '')
        return content

    context = "\n"
    for message in messages:
        logger.info(message)
        role = message.get("role")
        content = message.get("content")

        if isinstance(content, list):  # content: {'type': 'text', 'text': ''}
            content = content[-1].get('text', '')

        if role == "system" and ignore_system:
            continue

        context += f"{role}:\n{content}\n\n"

    return context


if __name__ == '__main__':
    messages = [
        {
            "role": "system",
            "content": [
                {
                    "type": "text",
                    "text": "你是数学家"
                }
            ]
        },
        {
            "role": "user",
            "content": [
                {
                    "type": "text",
                    "text": "1+1"
                }
            ]
        },
        {
            "role": "assistant",
            "content": [
                {
                    "type": "text",
                    "text": "2"
                }
            ]
        },

        {
            "role": "user",
            "content": [
                {
                    "type": "text",
                    "text": "1+2"
                }
            ]
        },

    ]

    print(oneturn2multiturn(messages, ignore_system=False))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : evn
# @Time         : 2023/4/27 16:57
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 

import os

# 环境变量配置
os.environ['JINA_HIDE_SURVEY'] = '1'
os.environ['TOKENIZERS_PARALLELISM'] = "true"
os.environ['MOMENTO_AUTH_TOKEN'] = 'eyJlbmRwb2ludCI6ImNlbGwtNC11cy13ZXN0LTItMS5wcm9kLmEubW9tZW50b2hxLmNvbSIsImFwaV9rZXkiOiJleUpoYkdjaU9pSklVekkxTmlKOS5leUp6ZFdJaU9pSXpNVE16TURNek1ETkFjWEV1WTI5dElpd2lkbVZ5SWpveExDSndJam9pUTBGQlBTSjkuSVRlX01ZRnBoVTBVb1I0SkdKOU92QXZ4dUJUOHBiQnpDWWlhZjlkZFYwdyJ9'

# LLM
os.environ['LLM_ROLE'] = '你扮演的角色是ChatLLM大模型，是由Betterme开发，基于ChatLLM-1000B模型训练的。'
os.environ['PROMPT_TEMPLATE'] = """
{role}
根据以下信息，简洁、专业地回答用户的问题。如果无法得到答案，请回复：“根据已知信息无法回答该问题”或“没有提供足够的信息”。请勿编造信息，答案必须使用中文。
已知信息：
{context}
问题：
{question}
""".strip() # Let's think step by step


# """
# {role}
# 请根据以下<>中的信息简洁、专业地回答问题。
# 信息：<{context}>
# 问题：{question}
# 如果无法从中得到答案，请回答“根据已知信息无法回答该问题”或“没有提供足够的信息”。请使用中文回答，不允许添加编造内容。
# """

# modelscope_cache
# PPNLP
if __name__ == '__main__':
    from pprint import pprint

    pprint(os.environ['PROMPT_TEMPLATE'])



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : images
# @Time         : 2024/11/13 15:44
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : https://fal.ai/models/fal-ai/flux-pro/v1.1-ultra/api

from meutils.pipe import *
from meutils.str_utils import parse_url, validate_url
from meutils.io.files_utils import to_url, to_url_fal

from meutils.schemas.image_types import ImageRequest, FluxImageRequest, SDImageRequest, ImagesResponse
from meutils.schemas.fal_types import FEISHU_URL
from meutils.config_utils.lark_utils import get_next_token_for_polling, get_series

from meutils.apis.translator import deeplx

from fal_client.client import AsyncClient, SyncClient, Status, FalClientError

recraft_styles = "any, realistic_image, digital_illustration, vector_illustration, realistic_image/b_and_w, realistic_image/hard_flash, realistic_image/hdr, realistic_image/natural_light, realistic_image/studio_portrait, realistic_image/enterprise, realistic_image/motion_blur, realistic_image/evening_light, realistic_image/faded_nostalgia, realistic_image/forest_life, realistic_image/mystic_naturalism, realistic_image/natural_tones, realistic_image/organic_calm, realistic_image/real_life_glow, realistic_image/retro_realism, realistic_image/retro_snapshot, realistic_image/urban_drama, realistic_image/village_realism, realistic_image/warm_folk, digital_illustration/pixel_art, digital_illustration/hand_drawn, digital_illustration/grain, digital_illustration/infantile_sketch, digital_illustration/2d_art_poster, digital_illustration/handmade_3d, digital_illustration/hand_drawn_outline, digital_illustration/engraving_color, digital_illustration/2d_art_poster_2, digital_illustration/antiquarian, digital_illustration/bold_fantasy, digital_illustration/child_book, digital_illustration/child_books, digital_illustration/cover, digital_illustration/crosshatch, digital_illustration/digital_engraving, digital_illustration/expressionism, digital_illustration/freehand_details, digital_illustration/grain_20, digital_illustration/graphic_intensity, digital_illustration/hard_comics, digital_illustration/long_shadow, digital_illustration/modern_folk, digital_illustration/multicolor, digital_illustration/neon_calm, digital_illustration/noir, digital_illustration/nostalgic_pastel, digital_illustration/outline_details, digital_illustration/pastel_gradient, digital_illustration/pastel_sketch, digital_illustration/pop_art, digital_illustration/pop_renaissance, digital_illustration/street_art, digital_illustration/tablet_sketch, digital_illustration/urban_glow, digital_illustration/urban_sketching, digital_illustration/vanilla_dreams, digital_illustration/young_adult_book, digital_illustration/young_adult_book_2, vector_illustration/bold_stroke, vector_illustration/chemistry, vector_illustration/colored_stencil, vector_illustration/contour_pop_art, vector_illustration/cosmics, vector_illustration/cutout, vector_illustration/depressive, vector_illustration/editorial, vector_illustration/emotional_flat, vector_illustration/infographical, vector_illustration/marker_outline, vector_illustration/mosaic, vector_illustration/naivector, vector_illustration/roundish_flat, vector_illustration/segmented_colors, vector_illustration/sharp_contrast, vector_illustration/thin, vector_illustration/vector_photo, vector_illustration/vivid_shapes, vector_illustration/engraving, vector_illustration/line_art, vector_illustration/line_circuit, vector_illustration/linocut"

ideogram_styles = "auto, general, realistic, design, render_3D, anime"

# https://fal.ai/models/fal-ai/any-llm/playground
llms = "anthropic/claude-3.5-sonnet, anthropic/claude-3-5-haiku, anthropic/claude-3-haiku, google/gemini-pro-1.5, google/gemini-flash-1.5, google/gemini-flash-1.5-8b, meta-llama/llama-3.2-1b-instruct, meta-llama/llama-3.2-3b-instruct, meta-llama/llama-3.1-8b-instruct, meta-llama/llama-3.1-70b-instruct, openai/gpt-4o-mini, openai/gpt-4o, deepseek/deepseek-r1"


@alru_cache(ttl=300)
async def check(token, threshold: float = 0):
    try:
        # data = await AsyncClient(key=token).upload(b'', '', '')

        data = await AsyncClient(key=token).run(
            "fal-ai/any-llm",
            arguments={
                "model": "meta-llama/llama-3.2-1b-instruct",
                "prompt": "1+1=",
                "max_tokens": 1,
            },
        )
        logger.debug(data)
        return True
    except Exception as exc:
        logger.error(exc)
        return False


async def generate(request: ImageRequest, api_key: Optional[str] = None):
    """https://fal.ai/models/fal-ai/flux-pro/v1.1-ultra/api#api-call-submit-request
    """
    logger.debug(request)

    s = time.time()
    token = api_key or await get_next_token_for_polling(feishu_url=FEISHU_URL, from_redis=True, check_token=check)

    arguments = request.model_dump(exclude_none=True)
    width, height = (request.size or "1024x1024").split("x")
    if request.model.startswith("fal-ai/recraft"):  # https://fal.ai/models/fal-ai/recraft-v3/api#queue-submit
        arguments = {
            **arguments,
            "image_size": {
                "width": width,
                "height": height
            },
            "style": request.style if str(request.style) in recraft_styles else "realistic_image",

            "output_format": "png",
        }
    elif request.model.startswith("fal-ai/flux/dev/image-to-image"):
        urls = parse_url(request.prompt)
        image_url = urls[-1]
        prompt = request.prompt.replace(image_url, "")

        arguments = {
            **arguments,
            "image_size": {
                "width": width,
                "height": height
            },
            "num_images": request.n or 1,

            "enable_safety_checker": False,
            "safety_tolerance": "6",
            "output_format": "png",
            "seed": request.seed,

            # image2image
            "image_url": image_url,
            "prompt": prompt,
            "guidance_scale": request.guidance or 3.5,

            **request.controls
        }
    elif "kontext" in request.model:  # https://fal.ai/models/fal-ai/flux-pro/kontext/max

        if (image_urls := request.image_urls) or (image_urls := parse_url(request.prompt)):
            if not validate_url(image_urls):
                # raise Exception(f"Invalid image url: {image_urls}")
                from fastapi import HTTPException, status
                raise HTTPException(
                    status_code=status.HTTP_451_UNAVAILABLE_FOR_LEGAL_REASONS,
                    detail=f"Invalid image url: {image_urls}",
                )

            for image_url in image_urls:
                request.prompt = request.prompt.replace(image_url, "")

            if len(image_urls) == 1:
                arguments['image_url'] = image_urls[0]
            else:
                request.model += "/multi"
                arguments['image_urls'] = image_urls
        else:
            request.model += "/text-to-image"

        # Path /kontext/text-to-image/text-to-image not found

        # fal-ai/flux-pro/kontext/max/text-to-image

        # fal-ai/flux-pro/kontext/max
        # fal-ai/flux-pro/kontext/max/multi

        # fal-ai/flux-pro/kontext/text-to-image

        # fal-ai/flux-pro/kontext
        # fal-ai/flux-pro/kontext/multi

        """
        {
          "prompt": "Put a donut next to the flour.",
          "guidance_scale": 3.5,
          "num_images": 1,
          "safety_tolerance": "2",
          "output_format": "jpeg",
          "image_url": "https://v3.fal.media/files/rabbit/rmgBxhwGYb2d3pl3x9sKf_output.png"
        }
        """

        arguments = {
            **arguments,

            "num_images": request.n or 1,

            "enable_safety_checker": False,
            "safety_tolerance": "5",
            "output_format": "png",
            "seed": request.seed,

            "guidance_scale": request.guidance or 3.5,

            "model": request.model,
            "prompt": await deeplx.llm_translate(request.prompt),

        }

        if request.aspect_ratio:
            arguments.pop("aspect_ratio", None)

            aspect_ratios = {'21:9', '16:9', '4:3', '3:2', '1:1', '2:3', '3:4', '9:16', '9:21'}

            if aspect_ratio := request.prompt2aspect_ratio(aspect_ratios):  # 提示词优先级最高
                arguments["aspect_ratio"] = aspect_ratio

            elif request.aspect_ratio in aspect_ratios:
                arguments["aspect_ratio"] = request.aspect_ratio

        logger.debug(bjson(arguments))

    elif request.model.startswith("fal-ai/flux"):  # https://fal.ai/models/fal-ai/flux-pro/v1.1-ultra/api

        arguments = {
            **arguments,
            "num_images": request.n or 1,

            "enable_safety_checker": False,
            "safety_tolerance": "6",
            "output_format": "png",
            "seed": request.seed,

            "aspect_ratio": request.aspect_ratio or "1:1"  # 21:9, 16:9, 4:3, 3:2, 1:1, 2:3, 3:4, 9:16, 9:21

        }

    elif request.model.startswith("fal-ai/ideogram"):
        arguments = {
            **arguments,
            "style": request.style if str(request.style) in ideogram_styles else "auto",

            # 10:16, 16:10, 9:16, 16:9, 4:3, 3:4, 1:1, 1:3, 3:1, 3:2, 2:3
            "aspect_ratio": request.aspect_ratio or "1:1",

            "seed": request.seed,

        }

    elif request.model.startswith("fal-ai/imagen4"):
        arguments = {
            **arguments,
            # 10:16, 16:10, 9:16, 16:9, 4:3, 3:4, 1:1, 1:3, 3:1, 3:2, 2:3
            "aspect_ratio": request.aspect_ratio or "1:1",

            "seed": request.seed,

        }

    elif request.model.startswith(("fal-ai/nano-banana", "fal-ai/gemini-25-flash-image")):
        arguments = {
            "prompt": request.prompt,
            "image_urls": request.image_urls,
            "num_images": request.n or 1,
            "output_format": "png"
        }
        if request.image_urls:
            request.model = f"""{request.model.removesuffix("/edit")}/edit"""

        logger.debug(bjson(arguments))

    try:

        data = await AsyncClient(key=token).run(
            application=request.model,
            arguments=arguments,
        )
        logger.debug(data)
        return ImagesResponse(data=data.get("images"), timings={"inference": time.time() - s})

    except Exception as exc:  #
        logger.error(exc)
        # from fastapi import HTTPException, status
        # raise HTTPException(
        #     status_code=status.HTTP_451_UNAVAILABLE_FOR_LEGAL_REASONS,
        #     detail=f"Prompt is sensitive.\n\n{request.prompt}\n\n{exc}",
        # )

        raise exc


if __name__ == '__main__':
    mapper = \
        {
            "flux-pro-1.1": "fal-ai/flux-pro/v1.1",
            "flux-pro-1.1-ultra": "fal-ai/flux-pro/v1.1-ultra",
            "ideogram-ai/ideogram-v2-turbo": "fal-ai/ideogram/v2/turbo",
            "ideogram-ai/ideogram-v2": "fal-ai/ideogram/v2",
            "recraftv3": "fal-ai/recraft-v3",

            "imagen4": "fal-ai/imagen4/preview",
            "flux-kontext-pro": "fal-ai/flux-pro/kontext",
            "flux-kontext-max": "fal-ai/flux-pro/kontext/max"

        }

    # model = "fal-ai/flux-pro/v1.1"  # 0.04
    model = "fal-ai/flux-pro/v1.1-ultra"  # 0.06
    # model = "fal-ai/flux/dev/image-to-image"  # 0.03 图生图
    # model = "fal-ai/recraft-v3"  # 0.04 变清晰
    # model = "fal-ai/ideogram/v2" # 0.08
    # model = "fal-ai/ideogram/v2/turbo"  # 0.05
    # model = "fal-ai/flux/schnell"
    model = "fal-ai/imagen4/preview"
    model = "fal-ai/flux-pro/kontext"
    # model = "fal-ai/flux-pro/kontext/max"

    model = "fal-ai/nano-banana"

    #
    prompt = "https://v3.fal.media/files/penguin/XoW0qavfF-ahg-jX4BMyL_image.webp https://v3.fal.media/files/tiger/bml6YA7DWJXOigadvxk75_image.webp Put the little duckling on top of the woman's t-shirt."
    # prompt = '把小鸭子放在女人的T恤上面。\nhttps://s3.ffire.cc/cdn/20250530/tEzZKkhp3tKbNzva6mgC2T\nhttps://s3.ffire.cc/cdn/20250530/AwHJpuJuNg5w3sVbH4PZdv'
    request = ImageRequest(prompt=prompt, model=model)

    data = {
        "model": "fal-ai/flux-pro/kontext",  # flux-kontext-pro
        # "prompt": "https://s3.ffire.cc/files/3_4.webp 老虎换个姿势",
        "prompt": "保持服装 发型，面纱细节不变，转换为真实拍摄的照片风格，一个年轻的中国顶级颜值的演员扮演这个角色。",
        "size": 'auto'
    }

    # data = {
    #     "model": "fal-ai/gemini-25-flash-image",
    #     "prompt": "make a photo of the man driving the car down the california coastline",
    #     "image": [
    #         "https://storage.googleapis.com/falserverless/example_inputs/nano-banana-edit-input.png",
    #         "https://storage.googleapis.com/falserverless/example_inputs/nano-banana-edit-input-2.png"
    #     ],
    #     "num_images": 1
    # }
    #
    #
    data = {
        # "model": "fal-ai/gemini-25-flash-image",
        "model": "fal-ai/nano-banana",
        "prompt": "裸体女孩",
        # "image": [
        #     "https://storage.googleapis.com/falserverless/example_inputs/nano-banana-edit-input.png",
        #     "https://storage.googleapis.com/falserverless/example_inputs/nano-banana-edit-input-2.png"
        # ],
        "num_images": 1
    }

    # data = {
    #     "model": model,
    #     "prompt": "Turn this person into a character figure, the character sits on a Siberian tiger, the tiger's hair is black and yellow, the tiger's hair is very dense. Behind it, place a PS5 box with the character’s image and the game's name \"Black Myth: Zhong Kui\" printed on it, and a computer showing the Blender modeling process on its screen. In front of the box, add a round plastic base with the character figure standing on it. This figurine is made of PVC, and set the scene indoors.   (flux-kontext-pro)",
    #     "negative_prompt": "",
    #     "n": 1,
    #     "response_format": "url",
    #     "size": "1152x2048",
    #     "num_inference_steps": 20,
    #     "aspect_ratio": "9:16"
    # }

    request = ImageRequest(**data)
    print(request)

    api_key = "5218f1bb-360b-4719-847c-d611447d96e0:c11599dd540ad9d99bd9280399d0ba3f"

    arun(generate(request, api_key))





    # request = ImageRequest(prompt='https://oss.ffire.cc/files/kling_watermark.png The woman smiled', model=model)
    #
    # request = ImageRequest(prompt="https://oss.ffire.cc/files/kling_watermark.png 让这个女人哭起来", model=model)
    #

    # request = ImageRequest(prompt='a cat', model=model)

    # token = "77fcfad2-aadc-4158-8e11-b30a5668dfad:ea5f38d864bde6707561d348436d2cea"
    # token = "5116af3f-1aa5-4c1b-b505-fd49f701dccc:7f4278d69d069cec4794fe658a09bd9d"

    # arun(generate(request))

    # arun(AsyncClient(key=token).key)

    # FEISHU_URL = "https://xchatllm.feishu.cn/sheets/QB6Psj8zrhB4JStEW0acI4iInNg?sheet=ef2e81"
    #
    # tokens = arun(get_series(FEISHU_URL))
    # tokens = ["578e143c-702d-46a1-8c02-d451bc3e155b:bb531a95f79c6026e7bc7ab676163c71"]
#     tokens = """
# 30e9fe5c-75d0-4bdd-b11d-ff17dbe4bba5:3de8535e2fa4d9347e599a845cf3e7b8
# 05c0b4f6-1443-4b28-aeff-771b65ebaebc:28df6b9b71430b05aca6b8bc7e873a4a
#     """.split()
#
#     r = []
#     for t in tokens:
#         if arun(check(t)):
#             r.append(t)
#
#     print('\n'.join(r))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : pixverse
# @Time         : 2024/7/24 13:14
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :
import uuid

from meutils.pipe import *
from meutils.decorators.retry import retrying
from meutils.schemas.pixverse_types import BASE_URL, PixverseRequest, EXAMPLES
from meutils.schemas.task_types import Task

from meutils.notice.feishu import send_message as _send_message
from meutils.config_utils.lark_utils import get_next_token_for_polling

FEISHU_URL = "https://xchatllm.feishu.cn/sheets/Bmjtst2f6hfMqFttbhLcdfRJnNf?sheet=f5db6a"

send_message = partial(
    _send_message,
    title=__name__,
    url="https://open.feishu.cn/open-apis/bot/v2/hook/dc1eda96-348e-4cb5-9c7c-2d87d584ca18"
)


# url = "https://app-api.pixverse.ai/creative_platform/media/getMediaDetail"
# "https://app-api.pixverse.ai/creative_platform/task/createVideoGenerateTask"
@retrying(max_retries=6, predicate=lambda r: not r)
async def create_task(request: Union[PixverseRequest, dict], token: Optional[str] = None):
    token = token or await get_next_token_for_polling(FEISHU_URL)
    ai_sign, token = token.split('|')

    headers = {
        'ai-sign': ai_sign,
        # 'ai-trace-id': '8a6cddbb-3cf0-4d44-a558-7ac777ab1f70', # uuid.uuid4()
        'token': token,
    }

    payload = request if isinstance(request, dict) else request.model_dump(exclude_none=True)

    async with httpx.AsyncClient(base_url=BASE_URL, headers=headers, timeout=30) as client:
        url = "/task/createImg2VideoGenerateTask" if payload.get("ImgUrl") else "/task/createVideoGenerateTask"
        response = await client.post(url, json=payload)

        logger.debug(response.status_code)
        logger.debug(response.text)

        if response.is_success:
            data = response.json()
            send_message([payload, data])
            return Task(status=1, data=data, system_fingerprint=token)

        else:
            # 触发重试
            if (
                    any(response.status_code == code for code in {401, 429})
                    # or any(i in response.text for i in {"enough credits"})
            ):
                return
            else:
                send_message(f"{response.text}\n\n{token}")

                return Task(status=0, data=response.text, system_fingerprint=token)


async def get_task(task_id: str, token: str):  # 1ee02f7b-42df-4926-b034-f56e4c4e2d31
    task_id = isinstance(task_id, str) and task_id.split("-", 1)[-1]
    ai_sign, token = token.split('|')

    payload = {
        "Type": 1,
        "MediaId": int(task_id)
    }

    headers = {
        'ai-sign': ai_sign,
        # 'ai-trace-id': '8a6cddbb-3cf0-4d44-a558-7ac777ab1f70', # uuid.uuid4()
        'token': token,
    }
    async with httpx.AsyncClient(base_url=BASE_URL, headers=headers, timeout=30) as client:
        response = await client.post(f"/media/getMediaDetail", json=payload)

        logger.debug(response.text)
        logger.debug(response.status_code)

        if response.is_success:
            return response.json()

        response.raise_for_status()


@retrying()  # 触发重试
async def upload(file: bytes, filename: Optional[str] = None, token: Optional[str] = None):  # 应该不绑定cookie
    ai_sign, token = token.split('|')

    filename = f"{uuid.uuid4()}.png"
    payload = {
        "images": [
            {
                "name": filename,
                "size": len(file),
                "path": f"upload/{filename}"
            }
        ]
    }
    headers = {
        # 'ai-sign': ai_sign,
        # 'ai-trace-id': '8a6cddbb-3cf0-4d44-a558-7ac777ab1f70',  # uuid.uuid4()
        'Token': token,
    }
    async with httpx.AsyncClient(base_url=BASE_URL, headers=headers, timeout=30) as client:
        response = await client.post("/getUploadToken")
        if response.is_success:
            resp = response.json().get("Resp")  # Ak Sk Token
            headers = {
                "Authorization": f"OSS {resp.get('Ak')}",
                'X-Oss-Security-Token': resp.get('Token')
            }
            # files = [('file', file_or_url)]
            oss_url = f"https://pixverse-fe-upload.oss-accelerate.aliyuncs.com/upload/{filename}"
            response = await client.put(oss_url, files={"file": file}, headers=headers)

            # response = await client.post("/media/batch_upload_media", json=payload)

            logger.debug(response.text)
            logger.debug(response.status_code)

            if response.is_success:
                logger.debug(response.text)
                return response.json()

            response.raise_for_status()


if __name__ == '__main__':
    token = "9d6332337b5d6976c3305ad3e0d8cd44eab6c0554d741a2fde856dff82f33cb7|eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9.eyJBY2NvdW50SWQiOjI4NjA1NDYzMzkyNDY3MiwiRXhwaXJlVGltZSI6MTcyMjQwMjI2OCwiVXNlcm5hbWUiOiIxODU1MDI4ODIzMyJ9.wiIBgmGACw8XTDf3eQ3jwGk33n8rl_hcceScasjKsIw"
    arun(get_task('286085978032192', token)) #

    # request = PixverseRequest(
    #     Prompt="开花过程",
    #     ImgUrl="https://dgss0.bdstatic.com/5bVWsj_p_tVS5dKfpU_Y_D3/res/r/image/2017-09-27/297f5edb1e984613083a2d3cc0c5bb36.png",
    #     # ImgUrl="https://media.pixverse.ai/upload%2F3597df25-8161-4eea-99a4-502ac2b49227.png"
    # )

    # arun(create_task(request))
    # file = Path('/Users/betterme/PycharmProjects/AI/test.png').read_bytes()
    # arun(upload(file, token=token))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : common
# @Time         : 2024/12/16 18:19
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : https://github.com/LLM-Red-Team/jimeng-free-api/commit/acd362a4cecd115938bf4bc9bbb0067738aa0d5b#diff-e6a7354ac1431dc3751e91efaf1799200c1ce2fa8abe975a49c32644290988baR121
from openai import AsyncClient

from meutils.pipe import *
from meutils.hash_utils import md5
from meutils.schemas.openai_types import TTSRequest

from meutils.schemas.jimeng_types import BASE_URL_GLOBAL as BASE_URL, MODELS_MAP, FEISHU_URL
from meutils.caches.redis_cache import cache

from fake_useragent import UserAgent

ua = UserAgent()


@lru_cache()
def get_headers(url, token: str = "693701c43e477b7c405cc7e2fef0ddbd"):
    device_time = f"{int(time.time())}"
    sign = md5(
        # f"9e2c|receive|7|5.8.0|{device_time}||11ac"
        f"9e2c|{url[-7:]}|7|5.8.0|{device_time}||11ac"
    )

    headers = {
        'appid': '513695',
        'appvr': '5.8.0',
        'device-time': device_time,
        'pf': '7',
        'sign': sign,
        'sign-ver': '1',
        'Cookie': f'sid_guard={token}|{device_time}|5184000|Fri,+14-Feb-2025+00:51:51+GMT',
        'User-Agent': ua.random,

        'content-type': 'application/json',
        # "Referer": "https://jimeng.jianying.com/ai-tool/image/generate",
    }
    return headers


@cache(ttl=3600 // 2)
async def get_upload_token(token, biz: Optional[str] = None):  # 3600 跨账号？

    if biz == "video":
        url = "/mweb/v1/get_upload_token"  # ?aid=513695&da_version=3.2.0&aigc_features=app_lip_sync

        payload = {"scene": 2}
        client = AsyncClient(base_url=BASE_URL)
        response = await client.post(url, body=payload, cast_to=object)
        logger.debug(bjson(response))
        return response

    url = "/artist/v2/tools/get_upload_token"
    headers = get_headers(url, token)

    payload = {"scene": 2}
    client = AsyncClient(base_url=BASE_URL, default_headers=headers)
    response = await client.post(url, body=payload, cast_to=object)
    logger.debug(bjson(response))
    return response


@alru_cache(12 * 3600)
async def receive_credit(token):
    # token = "eb4d120829cfd3ee957943f63d6152ed"  # y
    # token = "9ba826acc1a4bf0e10912eb01beccfe0"  # w
    url = "/commerce/v1/benefits/credit_receive"
    headers = get_headers(url, token)
    payload = {"time_zone": "Asia/Shanghai"}
    client = AsyncClient(base_url=BASE_URL, default_headers=headers)
    response = await client.post(url, body=payload, cast_to=object)
    logger.debug(bjson(response))


async def get_credit(token):
    # token = "eb4d120829cfd3ee957943f63d6152ed"  # y
    # token = "9ba826acc1a4bf0e10912eb01beccfe0"  # w

    url = "/commerce/v1/benefits/user_credit"
    headers = get_headers(url, token)

    payload = {}
    client = AsyncClient(base_url=BASE_URL, default_headers=headers)
    response = await client.post(url, body=payload, cast_to=object)
    if response['data']['credit']['gift_credit'] == 0:  # 签到
        await receive_credit(token)

    logger.debug(bjson(response))
    return response


async def check_token(token, threshold: int = 1):  # todo: 失效还是没积分
    try:
        response = await get_credit(token)
        logger.debug(bjson(response))
        # logger.error(f"{token}")

        credits = sum(response['data']['credit'].values())
        return credits >= threshold
    except Exception as e:
        logger.error(e)
        return False


def create_photo_lip_sync(request: TTSRequest):
    # 映射
    tts_info = {
        "name": request.voice or "魅力姐姐",
        "text": request.input,  # "永远相信美好的事情即将发生"
        "speed": request.seed or 1,
        "source_type": "text-to-speech",
        "tone_category_id": 0,
        "tone_category_key": "all",
        "tone_id": "7382879492752019995",
        "toneKey": request.voice or "魅力姐姐",
    }
    payload = {
        "promptSource": "photo_lip_sync",
        "generateTimes": 1,
        "lipSyncInfo": tts_info,
        "isUseAiGenPrompt": False,
        "batchNumber": 1
    }

    return {
        "submit_id": "",
        "task_extra": json.dumps(payload),
        "http_common_info": {
            "aid": 513695
        },
        "input": {
            "seed": 1834980836,
            "video_gen_inputs": [
                {
                    "v2v_opt": {},
                    "i2v_opt": {
                        "realman_avatar": {
                            "enable": True,
                            "origin_image": {
                                "width": 480,
                                "height": 270,
                                "image_uri": "tos-cn-i-tb4s082cfz/fae8f746367b40f5a708bf0f1e84de11",
                                "image_url": ""  ########## https://oss.ffire.cc/files/kling_watermark.png
                            },
                            "resource_id_loopy": "34464cac-5e3e-46be-88ff-ccbc7da4d742",
                            "resource_id_std": "2f66b408-37c3-4de5-b8a4-ba15c104612e",
                            "origin_audio": {
                                "duration": 3.216,
                                "vid": "v02bc3g10000ctrpqrfog65purfpn7a0"
                            },
                            "tts_info": json.dumps(tts_info)
                        }
                    },
                    "audio_vid": "v02bc3g10000ctrpqrfog65purfpn7a0",
                    "video_mode": 4
                }
            ]
        },
        "mode": "workbench",
        "history_option": {},
        "commerce_info": {
            "resource_id": "generate_video",
            "resource_id_type": "str",
            "resource_sub_type": "aigc",
            "benefit_type": "lip_sync_avatar_std"
        },
        "scene": "lip_sync_image",
        "client_trace_data": {},
        "submit_id_list": [
            "d4554e2c-707f-4ebe-b4e6-93aa6687d1c1"
        ]
    }


if __name__ == '__main__':
    from meutils.config_utils.lark_utils import aget_spreadsheet_values, get_series

    token = "693701c43e477b7c405cc7e2fef0ddbd"
    token = "eb4d120829cfd3ee957943f63d6152ed"
    token = "dcf7bbc31faed9740b0bf748cd4d2c74"
    token = "38d7d300b5e0a803431ef88d8d2acfef"
    token = "916fed81175f5186a2c05375699ea40d"
    token = "7c5e148d9fa858e3180c42f843c20454"
    token = "1c21a9fe6a4230609d7ff13e5cec41ec"
    token = "34438eb03d165737122180caf62a8058"
    token = "ffeee346fbd19eceebb79a7bfbca4bfe"
    # token = "b8bb4cb67dba6c0d1048bdc0596bc461"
    # token = "34438eb03d165737122180caf62a8058"
    # token = "a521dd578adcfb191fad38dd4baab498"
    token = "7d9969ffd8ad2edda7da8fff11cb9434"
    token = "1513337bdba08a1a77fedad95c03bc6c"
    token = "b1cd6317e4d161bbb3889b9defd769ff"
    token = "176b9aba6b81256b50abf08526cf311a"
    arun(check_token(token))

    # arun(get_upload_token(token, 'video'))

    # print(arun(aget_spreadsheet_values(feishu_url=FEISHU_URL, to_dataframe=True))[0].tolist())
    # tokens = arun(get_series(FEISHU_URL))
    # arun(get_credit(token))
    # arun(check_token(token))
    # _ = []
    # for token in tokens:
    #     if not arun(check_token(token)):
    #         logger.debug(f"无效 {token}")
    #         _.append(token)

    # arun(get_upload_token(token))
    #
    # request = ImageRequest(prompt='https://oss.ffire.cc/files/kling_watermark.png笑起来')
    # arun(create_draft_content(request, token))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : openai_types
# @Time         : 2024/6/7 17:30
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : https://github.com/Calcium-Ion/new-api/tree/main/dto

from meutils.pipe import *

from openai.types.chat import ChatCompletion as _ChatCompletion, ChatCompletionChunk as _ChatCompletionChunk
from openai.types.chat.chat_completion import Choice as _Choice, ChatCompletionMessage as _ChatCompletionMessage, \
    CompletionUsage as _CompletionUsage
from openai.types.chat.chat_completion_chunk import Choice as _ChunkChoice, ChoiceDelta
from openai._types import FileTypes
from openai.types import ImagesResponse as _ImagesResponse
from openai.types.shared_params import FunctionDefinition
from openai.types.chat import ChatCompletionToolParam

TOOLS = [
    {"type": "web_browser"},
    {"type": "code_interpreter"},
    {"type": "drawing_tool"},
]

BACKUP_MODEL = os.getenv("BACKUP_MODEL", "glm-4")


class Tool(BaseModel):
    # {"id": "", "type": "web_browser", "function": {}} => {"type": "web_browser", "function": None}
    function: Optional[dict] = None  # FunctionDefinition
    type: str

    def __init__(self, /, **data: Any):
        super().__init__(**data)
        self.function = self.function or None


class CompletionUsage(_CompletionUsage):
    prompt_tokens: int = 1000
    completion_tokens: Optional[int] = None
    total_tokens: Optional[int] = None

    def __init__(self, /, **data: Any):
        super().__init__(**data)
        self.completion_tokens = self.completion_tokens or 0
        self.total_tokens = self.total_tokens or self.prompt_tokens + self.completion_tokens


class ChatCompletionMessage(_ChatCompletionMessage):
    role: Literal["assistant"] = "assistant"
    """The role of the author of this message."""


class Choice(_Choice):
    index: int = 0
    finish_reason: Literal["stop", "length", "tool_calls", "content_filter", "function_call"] = None


class ChatCompletion(_ChatCompletion):
    id: str = Field(default_factory=lambda: f"chatcmpl-{shortuuid.random()}")
    created: int = Field(default_factory=lambda: int(time.time()))
    model: str = ""
    object: str = "chat.completion"
    usage: CompletionUsage = CompletionUsage()


class ChunkChoice(_ChunkChoice):
    index: int = 0


class ChatCompletionChunk(_ChatCompletionChunk):
    id: str = Field(default_factory=lambda: f"chatcmpl-{shortuuid.random()}")
    created: int = Field(default_factory=lambda: int(time.time()))
    model: str = ""
    object: str = "chat.completion.chunk"


chat_completion = ChatCompletion(
    choices=[Choice(message=ChatCompletionMessage(reasoning_content="", content=""))]
)
chat_completion_chunk = ChatCompletionChunk(
    choices=[ChunkChoice(delta=ChoiceDelta(reasoning_content="", content=""))]
)
chat_completion_chunk_stop = ChatCompletionChunk(
    choices=[ChunkChoice(delta=ChoiceDelta(reasoning_content="", content=""), finish_reason="stop")]
)


# chat_completion.choices[0].message.content = "*"
# chat_completion_chunk.choices[0].delta.content = "*"

class CompletionRequest(BaseModel):
    """
    prompt_filter_result.content_filter_results
    choice.content_filter_results

    todo: ['messages', 'model', 'frequency_penalty', 'function_call', 'functions', 'logit_bias', 'logprobs', 'max_tokens', 'n', 'presence_penalty', 'response_format', 'seed', 'stop', 'stream', 'temperature', 'tool_choice', 'tools', 'top_logprobs', 'top_p', 'user']
    """
    model: str = ''  # "gpt-3.5-turbo-file-id"

    # [{'role': 'user', 'content': 'hi'}]
    # [{'role': 'user', 'content':  [{"type": "text", "text": ""}]]
    # [{'role': 'user', 'content': [{"type": "image_url", "image_url": ""}]}] # 也兼容文件
    # [{'role': 'user', 'content': [{"type": "image_url", "image_url": {"url": ""}}]}] # 也兼容文件
    # [{'role': 'user', 'content':  [{"type": "file", "file_url": ""}]]
    messages: Optional[List[Dict[str, Any]]] = None  # 标准化
    metadata: Optional[Dict[str, str]] = None

    stream: Optional[bool] = False
    stream_options: Optional[dict] = None

    top_p: Optional[float] = 0.7
    temperature: Optional[float] = 0.7

    n: Optional[int] = 1
    max_tokens: Optional[int] = None
    stop: Optional[Union[str, List[str]]] = None

    frequency_penalty: Optional[float] = None
    user: Optional[str] = None

    reasoning_effort: Optional[Literal["low", "medium", "high"]] = None

    # Oneapi https://github.com/QuantumNous/new-api/blob/main/dto/openai_request.go
    extra_body: Optional[Any] = None
    enable_thinking: Optional[bool] = None  # ali
    thinking: Optional[dict] = None  # doubao "type": "disabled" "enabled" "auto"

    def __init__(self, **kwargs):
        super().__init__(**kwargs)

    @cached_property
    def system_instruction(self):
        # [{'role': 'system', 'content':  [{"type": "text", "text": ""}]]

        if message := self.messages and self.messages[0]:
            if message.get("role") == "system":
                return str(message.get("content", "")) or None

    @cached_property
    def last_message(self):
        return self.messages and self.messages[-1]

    @cached_property
    def last_user_content(self) -> str:
        """text"""
        for i, message in enumerate(self.messages[::-1], 1):
            if message.get("role") == "user":
                contents = message.get("content")
                if isinstance(contents, list):
                    for content in contents:
                        if content.get("type") == "text":
                            return content.get('text', "")

                else:
                    return str(contents)
        return ""

    @cached_property
    def last_assistant_content(self) -> str:
        """text"""
        for i, message in enumerate(self.messages[::-1], 1):
            if message.get("role") == "assistant":
                contents = message.get("content")
                if isinstance(contents, list):
                    for content in contents:
                        if content.get("type") == "text":
                            return content.get('text', "")
                else:
                    return str(contents)
        return ""

    @cached_property
    def last_urls(self):  # file_url 多轮对话需要  sum(request.last_urls.values(), [])
        """最新一轮的 user url 列表"""
        content_types = {
            "image_url",

            "file", "file_url",

            "audio_url", "input_audio",

            "video_url",
        }
        for i, message in enumerate(self.messages[::-1], 1):
            data = {}
            if message.get("role") == "user":  # 每一轮还要处理
                user_contents = message.get("content")
                if isinstance(user_contents, list):  # 用户 url
                    for content in user_contents:
                        content_type = content.get("type")
                        if content_type in content_types:
                            # logger.debug(content)
                            if _url := content.get(content_type, {}):  # {"type": "file", "file": fileid}
                                if isinstance(_url, str):  # 兼容了spark qwenai
                                    url = _url
                                else:
                                    url = _url.get("url") or _url.get("data")
                                url and data.setdefault(content_type, []).append(url)

            if data: return data
        return {}

    # def create_message(self, text: str, content_type: Optional[str] = None):
    #     """
    #     消息生成器
    #     :param text:
    #     :param content_type:
    #     :return:
    #     """
    #     message = {
    #         'role': 'user',
    #         'content': [
    #             {
    #                 "type": "text",
    #                 "text": text
    #             },
    #         ]
    #     }
    #
    #     return message

    class Config:
        extra = "allow"

        json_schema_extra = {
            "examples": [
                {
                    "model": "deepseek-chat",
                    "messages": [
                        {
                            "role": "user",
                            "content": "hi"
                        }
                    ],
                    "stream": True
                },
            ]
        }


class ChatCompletionRequest(BaseModel):
    """
    prompt_filter_result.content_filter_results
    choice.content_filter_results

    todo: ['messages', 'model', 'frequency_penalty', 'function_call', 'functions', 'logit_bias', 'logprobs', 'max_tokens', 'n', 'presence_penalty', 'response_format', 'seed', 'stop', 'stream', 'temperature', 'tool_choice', 'tools', 'top_logprobs', 'top_p', 'user']
    """
    model: str = ''  # "gpt-3.5-turbo-file-id"

    # [{'role': 'user', 'content': 'hi'}]
    # [{'role': 'user', 'content':  [{"type": "text", "text": ""}]]
    # [{'role': 'user', 'content': [{"type": "image_url", "image_url": ""}]}] # 也兼容文件
    # [{'role': 'user', 'content': [{"type": "image_url", "image_url": {"url": ""}}]}] # 也兼容文件
    # [{'role': 'user', 'content':  [{"type": "file", "file_url": ""}]]
    messages: Optional[List[Dict[str, Any]]] = None

    stream: Optional[bool] = False
    stream_options: Optional[dict] = None

    top_p: Optional[float] = 0.7
    temperature: Optional[float] = 0.7

    n: Optional[int] = 1
    max_tokens: Optional[int] = None
    stop: Optional[Union[str, List[str]]] = None

    presence_penalty: Optional[float] = 0.0
    frequency_penalty: Optional[float] = None
    user: Optional[str] = None

    # tools
    response_format: Optional[Any] = None
    function_call: Optional[Any] = None
    functions: Optional[Any] = None
    tools: Optional[List[Tool]] = None  # 为了兼容 oneapi
    tool_choice: Optional[Any] = None
    parallel_tool_calls: Optional[Any] = None

    # 拓展字段
    system_messages: Optional[list] = None
    last_content: Optional[Any] = None
    urls: List[str] = []

    system_fingerprint: Optional[str] = "🔥"

    def __init__(self, **kwargs):
        super().__init__(**kwargs)

        self.messages = self.messages or [{'role': 'user', 'content': 'hi'}]
        self.system_messages = [m for m in self.messages if m.get("role") == "system"]

        # 移除 不支持的 content type: 优雅处理多轮

        last_message = self.messages[-1]
        self.last_content = last_message.get("content", "")  # 大部分时间等价于user_content
        """[{'role': 'user', 'content':  [{"type": "text", "text": ""}]]"""
        if isinstance(self.last_content, list) and self.last_content:
            # 多模态 'text','image_url','video_url' and 'video'
            urls = (
                    jsonpath.jsonpath(self.last_content, expr='$..url')
                    or jsonpath.jsonpath(self.last_content, expr='$..image_url')
                # or jsonpath.jsonpath(self.last_content, expr='$..audio_url')
                # or jsonpath.jsonpath(self.last_content, expr='$..video_url')
                # or jsonpath.jsonpath(self.last_content, expr='$..file_url')
            )
            self.urls = self.urls or urls or []

            # 取最后一个文本提示词
            texts = jsonpath.jsonpath(self.last_content, expr='$..text') or []
            self.last_content = '\n'.join(texts)

        # 兼容 glm-4
        self.top_p = self.top_p is not None and np.clip(self.top_p, 0.01, 0.99)
        self.temperature = self.temperature is not None and np.clip(self.temperature, 0.01, 0.99)

        if self.model.startswith('o1'):
            self.top_p = 1
            self.temperature = 1

        if self.max_tokens:
            self.max_tokens = min(self.max_tokens, 4096)

    class Config:
        extra = "allow"

        json_schema_extra = {
            "examples": [
                {
                    "model": "gpt-3.5-turbo",
                    "messages": [
                        {
                            "role": "user",
                            "content": "hi"
                        }
                    ],
                    "stream": False
                },

                {
                    "model": "gpt-3.5-turbo",
                    "messages": [
                        {
                            "role": "user",
                            "content": "请按照下面标题，写一篇400字的文章\n王志文说，一个不熟的人找你借饯，说明他已经把熟人借遍了。除非你不想要了，否则不要借"
                        }
                    ],
                    "stream": False
                },

                # url
                {
                    "model": "url-gpt-3.5-turbo",
                    "messages": [
                        {
                            "role": "user",
                            "content": "总结一下https://mp.weixin.qq.com/s/Otl45GViytuAYPZw3m7q9w"
                        }
                    ],
                    "stream": False
                },

                # rag
                {
                    "messages": [
                        {
                            "content": "分别总结这两篇文章",
                            "role": "user"
                        }
                    ],
                    "model": "gpt-3.5-turbo",
                    "stream": False,
                    "file_ids": ["cn2a0s83r07am0knkeag", "cn2a3ralnl9crebipv4g"]
                }

            ]
        }


class ImageRequest(BaseModel):
    prompt: str

    model: str = 'pro'
    n: int = 1
    quality: str = 'hd'
    response_format: Literal["url", "b64_json"] = "url"
    size: str = '1024x1024'  # 测试默认值
    # sd: 768x1024 1024x576
    style: Union[str, Literal["vivid", "natural"]] = "natural"

    # 拓展参数
    guidance_scale: float = Field(default=5, alias="guidance")

    # https://blog.csdn.net/qq_37508554/article/details/133975130
    num_inference_steps: int = Field(default=20, alias="steps")

    seed: Optional[int] = None
    negative_prompt: Optional[str] = None

    # 拓展字段
    nsfw_level: str = "2"  # 小于等于6
    url: Optional[str] = None
    image: Optional[str] = None  # url/base64

    prompt_enhancement: bool = False

    def __init__(self, /, **data: Any):
        super().__init__(**data)
        if self.model.lower().__contains__('stabilityai'):
            self.num_inference_steps = 25
            self.guidance_scale = 7.5
        elif any(i in self.model.lower() for i in {"turbo", "lightning"}):
            self.num_inference_steps = 4
            self.guidance_scale = 1

    class Config:
        # frozen = True
        populate_by_name = True

        json_schema_extra = {
            "examples": [
                {
                    "model": "stable-diffusion-3-medium",  # sd3
                    "prompt": "画条狗",
                },
            ]
        }


class ImagesResponse(_ImagesResponse):
    created: int = Field(default_factory=lambda: int(time.time()))


class TTSRequest(BaseModel):
    model: Optional[Union[str, Literal["tts-1", "tts-1-hd"]]] = 'tts'
    voice: Optional[Union[str, Literal["alloy", "echo", "fable", "onyx", "nova", "shimmer",
    "male", "femal",
    ]]] = ""

    input: str
    instructions: Optional[str] = None
    emotion: Optional[Literal[
        "happy", "angry", "surprise", "coldness", "disgust", "fear", "excited", "hate", "sad", "fearful", "disgusted", "surprised", "neutral"]] = None

    speed: Optional[float] = None

    response_format: Union[
        str, Literal["mp3", "opus", "aac", "flac", "wav", "pcm", "b64_json", "url", "hex"]] = "b64_json"

    class Config:
        extra = "allow"

        json_schema_extra = {
            "examples": [
                {
                    "model": "tts-1",
                    "voice": "7f92f8afb8ec43bf81429cc1c9199cb1",
                    "input": "你好，我是AI助手",
                }
            ]
        }


class STTRequest(BaseModel):  # ASR
    file: Union[bytes, str]
    model: str = "whisper-1"
    prompt: Optional[str] = None
    response_format: Literal["text", "srt", "verbose_json", "vtt"] = "text"
    language: Optional[str] = None
    temperature: Optional[float] = None
    timestamp_granularities: Optional[List[Literal["word", "segment"]]] = None


if __name__ == '__main__':
    pass

    # print(ChatCompletion(choices=[Choice(message=ChatCompletionMessage(content="ChatCompletion"))]))
    # print(ChatCompletionChunk(choices=[ChunkChoice(delta=ChoiceDelta(content="ChatCompletionChunk"))]))
    #
    # print(chat_completion)
    # print(chat_completion_chunk)
    # print(chat_completion_chunk_stop)

    # print(ChatCompletionRequest(temperature=0, top_p=1))
    # print(ChatCompletionRequest(temperature=1, top_p=0))

    # file = UploadFile(open("/Users/betterme/PycharmProjects/AI/ChatLLM/chatllm/api/routers/cocopilot.py", "rb"))
    # #
    # print(AudioRequest(file=file))
    # print(ImagesResponse(data=[]))
    # print(ChatCompletionRequest(tools=[
    #     {
    #         # "id": "",
    #         "type": "web_browser",
    #         # "function": {}
    #     }
    # ],
    #     max_tokens=None,
    # ).model_dump_json(indent=4))

    # 创建实例
    # req1 = ImageRequest(guidance_scale=5.0, prompt="画条狗")
    # req2 = ImageRequest(guidance=5.0, prompt="画条狗")
    #
    # print(req1.guidance_scale)  # 输出: 5.0
    # print(req2.guidance_scale)  # 输出: 5.0
    #
    # # 两种方式都可以访问这个值
    # print(req1.model_dump())  # 输出: {'guidance_scale': 5.0}
    # print(req1.model_dump(by_alias=True))  # 输出: {'guidance': 5.0}

    # class A(BaseModel):
    #     n: int = Field(1, ge=1, le=0)
    #
    #
    # print(A(n=11))
    messages = [
        {'role': 'user',
         'content': [{"type": "image_url", "image_url": "https://oss.ffire.cc/files/kling_watermark.png"}]},
        {'role': 'assistant', 'content': [{"type": "image_url", "image_url": "这是个图片链接"}]},

        {'role': 'assistant', 'content': [{"type": "image_url", "image_url": "这是个图片链接"}]},

        {'role': 'user', 'content': [
            {"type": "image_url", "image_url": {"url": "这是个图片链接1"}},
            {"type": "file_url", "file_url": {"url": "这是个file_url"}},
            {"type": "file_url", "file_url": {"url": "这是个file_url"}},
            {"type": "file", "file": "这是个fileid"},

            {"type": "audio_url", "audio_url": {"url": "这是个file_url"}},
            {"type": "video_url", "video_url": {"url": "这是个video_url"}}
        ]},

        {'role': 'assistant', 'content': [{"type": "image_url", "image_url": "这是个图片链接"}]},
        {'role': 'user', 'content': [
            {"type": "text", "text": "这是个文本"},
            {"type": "image_url", "image_url": "这是个图片链接"}
        ]},
    ]

    messages = [{'role': 'system',
                 'content': 'undefined\n Current date: 2025-03-13'},
                {'role': 'user',
                 'content': [{'type': 'text', 'text': '解读'},
                             {'type': 'image_url',
                              'image_url': 'https://ai.chatfire.cn/files/images/xx-1741850404348-339a12ba3.png'}]},
                {},
                {'role': 'user', 'content': '总结一下'}]

    messages = [{'role': 'system',
                 'content': 'undefined\n Current date: 2025-03-13'},
                {'role': 'user',
                 'content': [{'type': 'text', 'text': '解读'},
                             {'type': 'image_url',
                              'image_url': 'https://ai.chatfire.cn/files/images/xx-1741850404348-339a12ba3.png'}]},
                {},
                {'role': 'user', 'content': '总结一下'}]

    messages = [{'role': 'system',
                 'content': 'undefined\n Current date: 2025-03-13'},
                {'role': 'user',
                 'content': [{'type': 'text', 'text': '一句话总结'},
                             {'type': 'image_url',
                              'image_url': 'https://ai.chatfire.cn/files/images/uniacess删除-1741849647273-c3f3be340.txt'}]},
                {},
                {'role': 'user', 'content': '总结'}]

    messages = [{'role': 'system',
                 'content': 'undefined\n Current date: 2025-03-13'},
                {'role': 'user',
                 'content': [{'type': 'text', 'text': '一句话总结'},
                             {'type': 'image_url',
                              'image_url': 'https://ai.chatfire.cn/files/images/uniacess删除-1741853464016-b176842ad.txt'}]},
                {},
                {'role': 'user',
                 'content': [{'type': 'text', 'text': '一句话总结'},
                             {'type': 'image_url',
                              'image_url': 'https://ai.chatfire.cn/files/images/uniacess删除-1741853464016-b176842ad.txt'}]},
                {},
                {'role': 'assistant',
                 'content': [{'type': 'text', 'text': '一句话总结'},
                             {'type': 'image_url',
                              'image_url': 'https://ai.chatfire.cn/files/images/uniacess删除-1741853464016-b176842ad.txt'}]},
                {},
                {'role': 'user',
                 'content': [{'type': 'text', 'text': '一句话总结'},
                             {'type': 'image_url',
                              'image_url': 'https://ai.chatfire.cn/files/images/uniacess删除-1741853464016-b176842ad.txt'}]},
                {},
                {'role': 'user',
                 'content': [{'type': 'text', 'text': '一句话总结'},
                             {'type': 'image_url',
                              'image_url': 'https://ai.chatfire.cn/files/images/uniacess删除-1741853464016-b176842ad.txt'}]},
                {},
                {'role': 'user',
                 'content': [
                     {'type': 'image_url',
                      'image_url': 'https://ai.chatfire.cn/files/images/uniacess删除-1741853464016-b176842ad.txt'},
                     {
                         "type": "input_audio",
                         "input_audio": {
                             "data": "base64_audio",
                             "format": "wav"
                         }
                     },
                     {'type': 'text', 'text': '一句话总结'},

                 ],

                 },
                {},
                # {'role': 'user', 'content': '总结'}
                ]
    #
    # r = ChatCompletionRequest(model="gpt-3.5-turbo", messages=messages)
    # r.messages[-1]['content'] = [{"type": "image_url", "image_url": {"url": r.urls[-1]}}]
    # print(r)

    # print(chat_completion_chunk)
    # print(chat_completion)
    # print(chat_completion_chunk_stop)

    # print(CompletionRequest(messages=messages).last_urls)
    print(CompletionRequest(messages=messages).last_urls)

    # print(mesages)
    # print(CompletionRequest(messages=messages).last_assistant_content)

    # print(chat_completion_chunk)
    # print(chat_completion)

    # chat_completion_chunk.usage = dict(
    #     completion_tokens=10,
    #     prompt_tokens=10,
    #     total_tokens=20,
    # )
    #
    # print(chat_completion_chunk)

    # print(ImageRequest(prompt='xx'))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : task
# @Time         : 2024/5/9 15:41
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :

from meutils.pipe import *
from meutils.db.redis_db import redis_aclient
from fastapi import status
from fastapi import APIRouter, File, UploadFile, Query, Form, Response, Request, FastAPI
from fastapi.responses import JSONResponse

router = APIRouter()


@router.get("/redis/{task_id}")
@alru_cache(ttl=60)  # 缓存
async def get_task_from_redis(task_id: str):
    task_info = await redis_aclient.get(task_id)
    return task_info


class RedisRequest(BaseModel):
    task_id: str
    task_info: str
    ttl: Optional[int] = None


@router.post("/redis")
async def set_task_for_redis(requst: RedisRequest):
    await redis_aclient.set(requst.task_id, requst.task_info, ex=requst.ttl)
    # return JSONResponse(content="", status_code=status.HTTP_200_OK)



redis_aclient.response_callbacks


if __name__ == '__main__':
    from meutils.serving.fastapi import App

    app = App()
    app.include_router(router)
    app.run()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : tql-ANN.
# @File         : ANN
# @Time         : 2019-12-04 20:05
# @Author       : yuanjie
# @Email        : meutils@qq.com
# @Software     : PyCharm
# @Description  : https://shenchunxu.blog.csdn.net/article/details/102881115?utm_medium=distribute.pc_relevant.none-task-blog-2%7Edefault%7ECTRLIST%7Edefault-1.no_search_link&depth_1-utm_source=distribute.pc_relevant.none-task-blog-2%7Edefault%7ECTRLIST%7Edefault-1.no_search_link

# todo: 增加idmapping并提供在线服务


import faiss
from meutils.pipe import *


# df = pd.DataFrame(idx + dis/100)
# def func(x):
#     idx = np.floor(x)
#     dis = (x - idx)*100
#     return i2w.get(idx), dis
#
# dff = df.applymap(func)

class ANN(object):
    """Flat支持: https://github.com/Jie-Yuan/faiss_note/blob/master/4.Faiss%20indexes%20%E8%BF%9B%E9%98%B6%E6%93%8D%E4%BD%9C.ipynb
    恢复原数据
    从index中移除向量
    搜索距离范围内的向量
    拆分/合并index

    cpu_index.make_direct_map()
    cpu_index.reconstruct(0)
    ann.index.reconstruct
    ann.index.reconstruct_c
    ann.index.reconstruct_n
    ann.index.reconstruct_n_c
    ann.index.search_and_reconstruct
    ann.index.search_and_reconstruct_c


    数据集的大小
    在高效检索的index中，聚类是其中的基础操作，数据量的大小主要影响聚类过程。

    如果小于1M， 使用"...,IVFx,..."
    N是数据集中向量个数，x一般取值[4sqrt(N),16sqrt(N)],需要30x ～ 256x个向量的数据集去训练。

    如果在1M-10M，使用"...,IMI2x10,..."
    使用k-means将训练集聚类为2^10个类，但是执行过程是在数据集的两半部分独立执行，即聚类中心有2^(2*10)个。

    如果在10M-100M，使用"...,IMI2x12,..."

    一个簇至少39个样本：一般2的N次方，我们一般用2**14或者更小一点 >> topk
    经验公式 2 ** (np.log2(n) // 2 + 2)
    """

    def __init__(self):
        self.nogpu_index_factory = {'HNSW', 'SQ'}

        self.keys = None

    def train(self, data, index_factory='Flat', metric=0, noramlize=False):
        """

        :param data:
        :param index_factory: 'IVF1024,Flat'
            https://www.cnblogs.com/houkai/p/9316172.html
            https://blog.csdn.net/xiaoxu2050/article/details/84982478
            https://github.com/facebookresearch/faiss/wiki/Faiss-indexes
            https://github.com/liqima/faiss_note/blob/master/4.Faiss%20indexes%20IO%E5%92%8Cindex%20factory.ipynb
        :param metric:
            faiss.METRIC_BrayCurtis
            faiss.METRIC_Canberra
            faiss.METRIC_INNER_PRODUCT: L2之后内积≈cosine 0 默认内积
            faiss.METRIC_JensenShannon
            faiss.METRIC_L1
            faiss.METRIC_L2
            faiss.METRIC_Linf
            faiss.METRIC_Lp
        :return:
        """

        # if keys is not None:
        #     assert len(set(keys)) == len(data)
        #     # self.w2i = dict(zip(keys, range(len(data))))
        #     self.i2w = dict(zip(range(len(data)), keys))

        if noramlize:
            data = self.noramlize(data).astype('float32')

        dim = data.shape[1]
        assert data.dtype == 'float32', "TODO: np.array([]).astype('float32')"

        args = [dim, index_factory, metric] if metric is not None else [dim, index_factory]

        self.index = faiss.index_factory(*args)

        if faiss.get_num_gpus() > 0:
            if any(index_factory.__contains__(i) for i in self.nogpu_index_factory):
                print(f"Don't Support GPU: {index_factory}")
            else:
                # gpu_index_flat = faiss.index_cpu_to_gpu(res, 0, index_flat)
                self.index = faiss.index_cpu_to_all_gpus(self.index)

        print("Train ...")
        self.index.train(data)
        self.index.add(data)

        print(f"Ntotal: {self.index.ntotal}")

    def search(self, data, topK=10, nprobe=1, k_factor=1):
        """

        :param data:
        :param topK:
        :param nprobe: nprobe参数始终是调整速度和结果精度之间权衡的一种方式。
        :param k_factor:
        :return:
        """
        self.index.k_factor = k_factor  # 搜索阶段会首先选取 k_factor*topK，重排序
        self.index.nprobe = nprobe  # default nprobe is 1, try a few more

        # distances, indexs = self.index.search(data, topK)
        # df = pd.DataFrame(indexs + distances / 100)
        # if self.keys is None:
        #     df.index = self.keys

        return self.index.search(data, topK)

    def write_index(self, file_name="index_file.index"):
        # faiss.index_cpu_to_all_gpus(index)
        if faiss.get_num_gpus() > 0:
            index = faiss.index_gpu_to_cpu(self.index)  # gpu版本？
            faiss.write_index(index, file_name)
        else:
            faiss.write_index(self.index, file_name)

    def read_index(self, file_name="index_file.index"):
        index = faiss.read_index(str(file_name))
        # if faiss.get_num_gpus() > 0:
        #     index = faiss.index_cpu_to_gpu()
        # index_new = faiss.clone_index(index) # 复制索引
        return index

    def noramlize(self, x):
        if len(x.shape) > 1:
            return x / np.clip(x ** 2, 1e-12, None).sum(axis=1).reshape((-1, 1) + x.shape[2:]) ** 0.5
        else:
            return x / np.clip(x ** 2, 1e-12, None).sum() ** 0.5


if __name__ == '__main__':
    import numpy as np

    data = np.random.random((1000, 128)).astype('float32')

    ann = ANN()
    ann.train(data, noramlize=True)

    dis, idx = ann.search(data[:10])

    print(dis)
    print(idx)

    # ann.write_index()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : deepinfra
# @Time         : 2024/11/26 13:57
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 

from openai import AsyncOpenAI

from meutils.pipe import *
from meutils.llm.clients import AsyncOpenAI
from meutils.llm.openai_utils import to_openai_params
from meutils.io.files_utils import to_url

from meutils.config_utils.lark_utils import get_next_token_for_polling
from meutils.schemas.openai_types import STTRequest, TTSRequest

from meutils.schemas.gitee_types import FEISHU_URL, BASE_URL


async def text_to_speech(request: TTSRequest, api_key: Optional[str] = None):
    api_key = api_key or await get_next_token_for_polling(FEISHU_URL)

    data = to_openai_params(request)
    logger.debug(bjson(data))

    client = AsyncOpenAI(base_url=BASE_URL, api_key=api_key)
    response = await client.audio.speech.create(**data)

    if request.response_format == "url":
        url = await to_url(response.content, filename=f'{shortuuid.random()}.mp3')
        return {"audio": url}

    return response


if __name__ == '__main__':
    data = {
        "model": "CosyVoice2",
        "input": "根据 prompt audio url克隆音色",
        "prompt_wav_url": "https://s3.ffire.cc/files/jay_prompt.wav",
        "response_format": "url"
    }


    print(TTSRequest(**data))

    request = TTSRequest(
        model="CosyVoice2",
        input="你好呀",
        # voice="alloy",
        response_format="b64_json"
    )

    # arun(text_to_speech(request))
    # print(MODELS.values())



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : MeUtils.
# @File         : import_utils
# @Time         : 2022/9/15 下午4:16
# @Author       : yuanjie
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 

from meutils.pipe import *


def import_mains(dir, pattern='*.py', filter_prefix=('__', '.')):  # todo: 优化
    """加载文件夹下的所有app（递归）, 入口函数都是main"""

    app_home = Path(sys_path_append(dir))
    n = app_home.parts.__len__()

    main_map = {}
    for p in app_home.rglob(pattern):
        if str.startswith(p.name, filter_prefix):  # 跳过 filter_prefix
            continue

        home_parts = p.parts[n:]
        module_name = '.'.join(home_parts).strip('.py')
        module = importlib.import_module(module_name)

        if hasattr(module, 'main'):  # 含有main入口函数的
            main_map[f'{app_home.stem}.{module_name}'] = getattr(module, 'main')

    return main_map



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : MeUtils.
# @File         : baidu
# @Time         : 2022/10/26 下午3:00
# @Author       : yuanjie
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :

from meutils.pipe import *
from meutils.hash_utils import md5
from meutils.request_utils import request4retry


class Translator(object):

    def __init__(self, appid, secretKey):
        self.appid = appid
        self.secretKey = secretKey
        self.url = "http://api.fanyi.baidu.com/api/trans/vip/translate"

    @disk_cache(location='__translator_cache')
    def query(self, q='苹果', fromLang='auto', toLang='en'):
        salt = random.randint(32768, 65536)
        # 生成签名
        sign = md5(f"{self.appid}{q}{salt}{self.secretKey}")

        data = {
            "appid": self.appid,
            "q": q,
            "from": fromLang,
            "to": toLang,
            "salt": salt,
            "sign": sign,
        }

        r = request4retry(self.url, 'post', data=data)
        return r.get('trans_result')


if __name__ == '__main__':
    appid = '20190718000319131'  # 你的appid
    secretKey = '***'  # 你的密钥
    print(Translator(appid, secretKey).query('中国有很多很多人'))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : __init__.py
# @Time         : 2025/4/9 11:23
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 

from meutils.pipe import *



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : common
# @Time         : 2023/7/20 10:35
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 

from queue import Queue


class KeyQueue(object):
    """支持 redis 做队列"""

    def __init__(self, keys: list, queue=None, queue_name='__queue__'):
        self._queue = queue or Queue()
        self.queue_name = queue_name
        for key in keys:  # 初始化队列
            if isinstance(self._queue, Queue):
                self._queue.put(key)
            else:
                self._queue.rpush(queue_name, key)

    def get_and_put(self, n: int = 1):
        for _ in range(n):
            if isinstance(self._queue, Queue):
                element = self._queue.get()
                self._queue.put(element)
            else:
                element = self._queue.blpop(self.queue_name)
                self._queue.rpush(self.queue_name, element)

            yield element

    @property
    def queue(self):
        _ = self._queue.queue if isinstance(self._queue, Queue) else self._queue.lrange(self.queue_name, 0, -1)
        return list(_)


if __name__ == '__main__':
    kq = KeyQueue(list(range(10)), Queue())

    print(kq.queue)
    print(list(kq.get_and_put(1)))
    print(kq.queue)
    print(list(kq.get_and_put(2)))
    print(kq.queue)

    # import redis
    #
    # r = redis.Redis(host='localhost', port=6379, db=0, decode_responses=True)  # 集群版呢？
    #
    # kq = KeyQueue(list(range(10, 20)), queue=r)
    #
    # print(kq.queue)
    # print(list(kq.get_and_put(1)))
    # print(kq.queue)
    # print(list(kq.get_and_put(2)))
    # print(kq.queue)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : routers
# @Time         : 2023/8/4 14:18
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 

from celery.result import AsyncResult

from fastapi import FastAPI, APIRouter
from fastapi import FastAPI, Form, Depends, File, UploadFile, Body, Request, BackgroundTasks

from meutils.pipe import *
from meutils.serving.celery.tasks import *

router = APIRouter()


@router.get("/celery/{task_id}")
async def get_result(task_id):
    result = AsyncResult(id=task_id)
    if result.ready():
        return result.get()  # task_id
    else:
        return {'code': 1, 'message': result.status, 'task_id': task_id}


@router.post("/celery/task")  # producer
async def create(request: Request):
    kwargs = await request.json()
    task = proxy_task.delay(**kwargs)
    return {'code': 1, 'message': '', 'task_id': task.id}


if __name__ == '__main__':
    from meutils.serving.fastapi import App

    app = App()
    app.include_router(router)
    app.run(port=8899)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : x
# @Time         : 2024/11/14 15:26
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 

from meutils.pipe import *


class VendorInfo(BaseModel):
    name: str
    desc: Optional[str] = None
    icon: Optional[str] = None
    notice: Optional[str] = None

    class Config:
        # 允许额外字段，增加灵活性
        extra = 'allow'


if __name__ == '__main__':
    print(VendorInfo(name="1", xx="2"))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : sentence_transformers
# @Time         : 2023/7/4 16:11
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 

from meutils.pipe import *

from sentence_transformers import SentenceTransformer as _SentenceTransformer


class SentenceTransformer(_SentenceTransformer):

    def encode_multi(self, sentences: List[str], target_devices: List[str] = None, batch_size: int = 32,
                     chunk_size: int = None):
        pool = self.start_multi_process_pool(target_devices)
        self.encode_multi_process(sentences, pool, batch_size, chunk_size)


if __name__ == '__main__':
    model = SentenceTransformer('/Users/betterme/PycharmProjects/AI/embeddings/m3e-small')
    # model.encode = partial(model.encode, show_progress_bar=True)
    # pool = model.start_multi_process_pool()
    # model.encode_multi_process(['1']*100, pool)

    print(model.encode(['e']).shape)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : config
# @Time         : 2023/4/26 10:40
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :
import streamlit

from meutils.serving.streamlit import *

st.set_page_config(page_title='🔥ChatPDF', layout='wide', initial_sidebar_state='collapsed')


################################################################################################################
class Conf(BaseConfig):
    encode_model = 'nghuyong/ernie-3.0-nano-zh'
    llm = "THUDM/chatglm-6b"  # /Users/betterme/PycharmProjects/AI/CHAT_MODEL/chatglm
    cachedir = 'pdf_cache'

    topk: int = 3
    threshold: float = 0.66


conf = Conf()
conf = set_config(conf)

if st.session_state.get('init'):
    st.json(st.session_state)



streamlit.image('https://api.isoyu.com/qr/?m=1&e=L&p=8&url=https://vip.chatllm.vip/')
